@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: CustomServiceDialog.cpp,v 1.15 2004/06/17 06:37:00 vkurland Exp $
+  $Id: CustomServiceDialog.cpp,v 1.16 2004/12/01 07:44:52 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -80,7 +80,7 @@ void CustomServiceDialog::loadFWObject(FWObject *o)
         allCodes[ i.key() ]=s->getCodeForPlatform( i.key().latin1() ).c_str();
     }
 
-    code->setText( allCodes[showPlatform] );
+    code->setText( QString::fromUtf8(allCodes[showPlatform] ));
 
     apply->setEnabled( false );
 
@@ -107,7 +107,7 @@ void CustomServiceDialog::changed()
     if (!init)
     {
         QString  pl   = platformReverseMap[platform->currentText()];
-        allCodes[pl] = code->text().latin1();
+        allCodes[pl] = code->text().utf8();
 
         apply->setEnabled( true );
     }
@@ -132,7 +132,7 @@ void CustomServiceDialog::platformChanged()
 {
     init=true;
     QString npl = platformReverseMap[platform->currentText()];
-    code->setText( allCodes[ npl ] );
+    code->setText( QString::fromUtf8(allCodes[ npl ]) );
     showPlatform = npl;
     init=false;
 //    changed();
@@ -148,12 +148,18 @@ void CustomServiceDialog::applyChanges()
 
     string oldname=obj->getName();
     obj->setName( string(obj_name->text().utf8()) );
-    obj->setComment( string(comment->text().utf8()) );
-    
+    string commText = string(comment->text().utf8());
+    obj->setComment( commText );
+
     QMap<QString,QString>::iterator i;
     for (i=allCodes.begin(); i!=allCodes.end(); ++i)
-        s->setCodeForPlatform( i.key().latin1(), i.data().latin1() );
-
+    {
+        string code = string(i.data().utf8());
+        if (fwbdebug)
+            qDebug("Storing custom service code %s :: %s",
+                   i.key().latin1(),code.c_str());
+        s->setCodeForPlatform( i.key().latin1(), code );
+    }
     om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
  
     init=true;
