@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWObjectPropertiesFactory.cpp,v 1.23 2005/01/24 09:29:45 vkurland Exp $
+  $Id: FWObjectPropertiesFactory.cpp,v 1.25.4.1 2005/09/13 04:55:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -255,13 +255,24 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
             if (showPath && !tooltip) str += path + "<br>\n";
             Group *g=Group::cast(obj);
             str += QObject::tr("%1 objects<br>\n").arg(g->size());
-            for (FWObject::iterator i=g->begin(); i!=g->end(); ++i)
+            int n = 0;
+            list<FWObject*> ll = *g;
+            ll.sort(FWObjectNameCmpPredicate());
+
+            for (FWObject::iterator i=ll.begin(); i!=ll.end(); ++i,++n)
             {
-                FWObject *o1=*i;
-                if (FWReference::cast(o1)!=NULL)
-                    o1=FWReference::cast(o1)->getPointer();
-                str += QString(o1->getTypeName().c_str())
-                    + "  <b>" + QString::fromUtf8(o1->getName().c_str()) + "</b><br>\n";
+                if (n>20)  // arbitrary number
+                {
+                    str += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;.&nbsp;.&nbsp;.&nbsp;";
+                    break;
+                } else 
+                {
+                    FWObject *o1=*i;
+                    if (FWReference::cast(o1)!=NULL)
+                        o1=FWReference::cast(o1)->getPointer();
+                    str += QString(o1->getTypeName().c_str())
+                        + "  <b>" + QString::fromUtf8(o1->getName().c_str()) + "</b><br>\n";
+                }
             }
         } else if (Firewall::isA(obj))
         {
@@ -289,7 +300,7 @@ QString FWObjectPropertiesFactory::getObjectPropertiesDetailed(FWObject *obj,
             
             FWObject *p=obj;
             while (p!=NULL && !Firewall::isA(p)) p=p->getParent();
-            if (p!=NULL && p->getStr("platform")=="pix")
+            if (p!=NULL && (p->getStr("platform")=="pix" || p->getStr("platform")=="fwsm"))
             {
                 int sl=Interface::constcast(obj)->getSecurityLevel();
                 q=q+QString("sec.level %1").arg(sl);
