@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindowPrint.cpp,v 1.19 2005/04/07 04:31:47 vkurland Exp $
+  $Id: FWWindowPrint.cpp,v 1.19.4.1 2005/09/04 20:49:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -216,15 +216,18 @@ int addObjectsToTable(list<FWObject*> &objects,
 */
 
         QString descr = FWObjectPropertiesFactory::getObjectProperties(*i);
+        QString comment =  QString::fromUtf8((*i)->getComment().c_str());
 
         tbl->setText(row,col+1, descr);
+        tbl->setText(row,col+2, comment);
+        tbl->item(row,col+2)->setWordWrap(true);
 
         added++;
 
         if (fwbdebug)
             qDebug("objTbl: row=%d col=%d '%s'",row,col,(*i)->getName().c_str());
 
-        col = col+2;
+        col = col+3;
         if (col>=tbl->numCols())
         {
             col = 0;
@@ -580,7 +583,7 @@ void FWWindow::filePrint()
                 bool haveObjGroups = false;
                 bool haveSrvGroups = false;
 
-                QTable objTbl(1,4);
+                QTable objTbl(1,6);
                 objTbl.setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
                                                    (QSizePolicy::SizeType)7) );
                 objTbl.setFrameStyle(QFrame::NoFrame | QFrame::Plain);
@@ -791,7 +794,7 @@ void FWWindow::filePrint()
 
                     for (FWObject::iterator obj=groups.begin(); obj!=groups.end(); ++obj)
                     {
-                        QTable objTbl(1,4);
+                        QTable objTbl(1,6);
                         objTbl.setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
                                                            (QSizePolicy::SizeType)7) );
                         objTbl.setFrameStyle(QFrame::NoFrame | QFrame::Plain);
