@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FirewallDialog.cpp,v 1.27 2004/07/13 22:42:58 vkurland Exp $
+  $Id: FirewallDialog.cpp,v 1.29 2004/12/05 21:52:31 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -52,6 +52,7 @@
 #include <qmessagebox.h>
 #include <qpushbutton.h>
 #include <qiconview.h>
+#include <qtabwidget.h>
 
 #include <iostream>
 
@@ -66,6 +67,9 @@ void FirewallDialog::loadFWObject(FWObject *o)
 
     init=true;
 
+/* hide tab "Templates" */
+    dialogTabWidget->removePage( dialogTabWidget->page(1) );
+    
     fillLibraries(libs,obj);
 
 /* fill in platform */
@@ -168,13 +172,18 @@ void FirewallDialog::platformChanged()
     QString so=Resources::platform_res[readPlatform(platform).latin1()]->getResourceStr("/FWBuilderResources/Target/supported_os").c_str();
     if (so.isEmpty()) return;
 
-    cerr << "so=" << so << endl;
-
     QString ho=so.section(",",0);
-
-    cerr << "ho=" << ho << endl;
-
     setHostOS( hostOS, ho.latin1() );
+
+    QString pl = readPlatform(platform);
+    fwAdvanced->setEnabled( pl!="unknown" );
+}
+
+void FirewallDialog::hostOSChanged()
+{
+    changed();
+    QString ho = readHostOS(hostOS);
+    osAdvanced->setEnabled( ho!="unknown_os" );
 }
 
 void FirewallDialog::changed()
@@ -222,8 +231,6 @@ void FirewallDialog::applyChanges()
     obj->setComment( string(comment->text().utf8()) );
     mgmt->getSNMPManagement()->setReadCommunity( snmpCommunity->text().latin1() );
 
-    om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
-
     string pl = readPlatform(platform).latin1();
     obj->setStr("platform", pl );
 
@@ -231,6 +238,8 @@ void FirewallDialog::applyChanges()
 
     saveVersion();
 
+    om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
+
     if (oldplatform!=pl || oldname!=newname)  mw->reopenFirewall();
 
     init=true;
