@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: GroupObjectDialog.cpp,v 1.40 2004/07/10 06:22:18 vkurland Exp $
+  $Id: GroupObjectDialog.cpp,v 1.43 2005/04/17 21:58:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -255,6 +255,10 @@ void GroupObjectDialog::insertObject(FWObject *o)
 
     if ( ! g->validateChild(o) || g->isReadOnly() ) return;
 
+    if (fwbdebug)
+        qDebug("Adding object %s to the group %s",
+               o->getName().c_str(), g->getName().c_str());
+
 /* avoid duplicates */
     string cp_id=o->getId();
 
@@ -290,12 +294,14 @@ void GroupObjectDialog::addIcon(FWObject *o,bool ref)
             string("/FWBuilderResources/Type/") +
             o->getTypeName() + "/hidden") ) return;
 
+    QString obj_name=QString::fromUtf8(o->getName().c_str());
+
     QString icn_filename =
         Resources::global_res->getObjResourceStr(o, (ref)?"icon-ref":"icon").c_str();
 
     ObjectIconViewItem *ivitm = new ObjectIconViewItem(iconView,
-                                                      o->getName().c_str(),
-                                                      QPixmap::fromMimeSource( icn_filename ) );
+                                                       obj_name,
+                                                       QPixmap::fromMimeSource( icn_filename ) );
     ivitm->setProperty("id",   o->getId().c_str()   );
     ivitm->setProperty("type", o->getTypeName().c_str() );
     ivitm->setFWObject( o );
@@ -305,7 +311,7 @@ void GroupObjectDialog::addIcon(FWObject *o,bool ref)
     icn_filename = Resources::global_res->getObjResourceStr(o, "icon-tree").c_str();
 
     ObjectTreeViewItem *tvitm=new ObjectTreeViewItem( listView );
-    tvitm->setText( 0, o->getName().c_str() );
+    tvitm->setText( 0, obj_name );
     tvitm->setText( 1, FWObjectPropertiesFactory::getObjectProperties(o) );
     tvitm->setPixmap( 0, QPixmap::fromMimeSource( icn_filename ) );
     tvitm->setDragEnabled(true);
@@ -637,8 +643,16 @@ void GroupObjectDialog::cutObj()
 
 void GroupObjectDialog::pasteObj()
 {
-    if (FWObjectClipboard::obj_clipboard->getObject()==NULL) return;
-    insertObject( FWObjectClipboard::obj_clipboard->getObject() );
+    vector<FWObject*>::iterator i;
+    for (i= FWObjectClipboard::obj_clipboard->begin();
+         i!=FWObjectClipboard::obj_clipboard->end(); ++i)
+    {
+        FWObject *co= *i;
+        insertObject( co );
+    }
+
+//    if (FWObjectClipboard::obj_clipboard->getObject()==NULL) return;
+//    insertObject( FWObjectClipboard::obj_clipboard->getObject() );
 }
 
 void GroupObjectDialog::deleteObj()
