@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: HostDialog.cpp,v 1.15 2004/06/17 06:37:00 vkurland Exp $
+  $Id: HostDialog.cpp,v 1.16 2005/05/29 04:38:44 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -64,7 +64,7 @@ void HostDialog::loadFWObject(FWObject *o)
     FWOptions  *opt =s->getOptionsObject();
 
     obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
-    snmpCommunity->setText( mgmt->getSNMPManagement()->getReadCommunity().c_str() );
+//    snmpCommunity->setText( mgmt->getSNMPManagement()->getReadCommunity().c_str() );
     MACmatching->setChecked( opt->getBool("use_mac_addr_filter") );
 
     comment->setText( QString::fromUtf8(s->getComment().c_str()) );
@@ -77,8 +77,8 @@ void HostDialog::loadFWObject(FWObject *o)
     libs->setEnabled(!o->isReadOnly());
     setDisabledPalette(libs);
 
-    snmpCommunity->setEnabled(!o->isReadOnly());
-    setDisabledPalette(snmpCommunity);
+//    snmpCommunity->setEnabled(!o->isReadOnly());
+//    setDisabledPalette(snmpCommunity);
 
     MACmatching->setEnabled(!o->isReadOnly());
     setDisabledPalette(MACmatching);
@@ -126,7 +126,7 @@ void HostDialog::applyChanges()
     string oldname=obj->getName();
     obj->setName( string(obj_name->text().utf8()) );
     obj->setComment( string(comment->text().utf8()) );
-    mgmt->getSNMPManagement()->setReadCommunity( snmpCommunity->text().latin1() );
+//    mgmt->getSNMPManagement()->setReadCommunity( snmpCommunity->text().latin1() );
     opt->setBool("use_mac_addr_filter", MACmatching->isChecked() );
 
     om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
