@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: InterfaceDialog.cpp,v 1.21 2004/07/13 22:42:58 vkurland Exp $
+  $Id: InterfaceDialog.cpp,v 1.23 2005/03/21 06:36:47 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -91,10 +91,11 @@ void InterfaceDialog::loadFWObject(FWObject *o)
 
     FWObject *f=obj->getParent();
 
-/* if parent is a host, hide security level and netzone */
+/* if parent is a host, hide checkbox 'external', security level and netzone */
     if (Host::isA( f ))
     {
         ext->setEnabled( false );
+        ext->hide();
         seclevelFrame->hide();
         netzoneFrame->hide();
     }
@@ -248,6 +249,8 @@ void InterfaceDialog::changed()
 void InterfaceDialog::validate(bool *res)
 {
     *res=true;
+    if (!isTreeReadWrite(this,obj)) *res=false;
+    if (!validateName(this,obj,obj_name->text())) *res=false;
 }
 
 void InterfaceDialog::isChanged(bool *res)
@@ -322,7 +325,8 @@ void InterfaceDialog::discardChanges()
  */
 void InterfaceDialog::closeEvent(QCloseEvent *e)
 {
+    if (fwbdebug)
+        qDebug("InterfaceDialog::coseEvent  got close event: %p",e);
     emit close_sign(e);
-
 }
 
