@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectEditor.h,v 1.10 2004/06/06 20:45:49 vkurland Exp $
+  $Id: ObjectEditor.h,v 1.12 2004/12/11 07:54:30 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -52,6 +52,9 @@ class ObjectEditor : public QObject {
     libfwbuilder::FWObject          *opened;
     int                              visible;
 
+
+    void disconnectSignals();
+    
 public:
     
     ObjectEditor( QWidget *parent );
@@ -64,7 +67,7 @@ public:
     
     libfwbuilder::FWObject* getOpened() { return opened; }
 
-    bool validateDialog();
+    bool validateAndSave();
 
 public slots:
     void validateAndClose(QCloseEvent *e);
