@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.cpp,v 1.88 2004/09/28 05:21:00 vkurland Exp $
+  $Id: ObjectManipulator.cpp,v 1.102 2005/04/17 21:58:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -97,6 +97,7 @@
 #include "fwbuilder/IntervalGroup.h"
 
 #include <iostream>
+#include <algorithm>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -258,15 +259,8 @@ ObjectTreeViewItem* ObjectManipulator::insertObject( ObjectTreeViewItem *itm,
 void ObjectManipulator::insertSubtree( ObjectTreeViewItem *itm,
                                        FWObject *obj )
 {
-    if (fwbdebug)
-        qDebug("ObjectManipulator::insertSubtree itm=%p  obj: %s",
-               itm, obj->getName().c_str() );
-
     ObjectTreeViewItem *nitm = insertObject(itm, obj);
 
-    if (fwbdebug)
-        qDebug("ObjectManipulator::insertSubtree nitm=%p", nitm);
-
     if (nitm==NULL) return;
     if ( FWBTree::isSystem(obj) ) nitm->setOpen( st->getExpandTree() );
 
@@ -462,7 +456,7 @@ void ObjectManipulator::autorename(FWObject *obj,bool ask)
 "%1 will stay the same.)")
                 .arg(QString::fromUtf8(obj->getName().c_str()))
                 .arg(QString::fromUtf8(obj->getName().c_str())),
-                "&Yes", "&No", QString::null,
+                tr("&Yes"), tr("&No"), QString::null,
                 0, 1 )==0 )
         {
             list<FWObject*> il = obj->getByType(Interface::TYPENAME);
@@ -491,7 +485,7 @@ void ObjectManipulator::autorename(FWObject *obj,bool ask)
 "%1 will stay the same.)")
                 .arg(QString::fromUtf8(obj->getName().c_str()))
                 .arg(QString::fromUtf8(obj->getName().c_str())),
-                "&Yes", "&No", QString::null,
+                tr("&Yes"), tr("&No"), QString::null,
                 0, 1 )==0 )
         {
             autorename(obj,IPv4::TYPENAME,"ip");
@@ -581,6 +575,10 @@ void ObjectManipulator::loadObjects(FWObjectDatabase *db)
     {
         FWObject *lib = (*i);
 
+        if (fwbdebug)
+            qDebug("ObjectManipulator::loadObjects  lib %p %s %s",
+                   lib, lib->getId().c_str(), lib->getName().c_str() );
+
         if ( lib->getId()==DELETED_LIB &&
              ! st->getBool("UI/ShowDeletedObjects")) continue;
 
@@ -825,8 +823,8 @@ void ObjectManipulator::contextMenu(QListViewItem *item,
                                   SLOT( copyObj() ) );
     int cutID =popup->insertItem( tr("Cut") ,       this ,
                                   SLOT( cutObj() ) );
-    popup->insertItem( tr("Paste") ,     this ,
-                       SLOT( pasteObj() ) );
+    int pasteID =popup->insertItem( tr("Paste") ,     this ,
+                                    SLOT( pasteObj() ) );
 
     popup->insertSeparator();
 
@@ -848,7 +846,7 @@ void ObjectManipulator::contextMenu(QListViewItem *item,
         if (Interface::isA(currentObj) && ! currentObj->isReadOnly())
         {
             newID1=popup->insertItem( tr("Add IP Address"),    this ,
-                               SLOT( newAddress() ) );
+                               SLOT( newInterfaceAddress() ) );
             newID2=popup->insertItem( tr("Add MAC Address"),   this ,
                                SLOT( newPhysicalAddress() ) );
         }
@@ -934,72 +932,127 @@ void ObjectManipulator::contextMenu(QListViewItem *item,
     } else
         popup->setItemEnabled(edtID,  false);
 
-/* TODO: should really scan all selected objects to make sure conditions
- * are met for all of them, not only for currentObj
- */
-
-    bool f1=true;
-    bool f2=true;
-    bool f3=true;
-    bool f4=true;
-    bool f5=true;
-
+    bool dupMenuItem=true;
+    bool moveMenuItem=true;
+    bool copyMenuItem=true;
+    bool pasteMenuItem=true;
+    bool delMenuItem=true;
+    bool newMenuItem=true;
     bool inDeletedObjects = false;
 
-    FWObject *delObjLib = mw->db()->getById( FWObjectDatabase::getDeletedObjectsId());
+    getMenuState( (moveTargets>0),
+                  dupMenuItem,moveMenuItem,copyMenuItem,pasteMenuItem,
+                  delMenuItem,newMenuItem,inDeletedObjects);
+
+    popup->setItemEnabled(dupID, dupMenuItem); 
+    popup->setItemEnabled(movID, moveMenuItem);
+    popup->setItemEnabled(copyID,copyMenuItem);
+    popup->setItemEnabled(pasteID,pasteMenuItem);
+    
+    popup->setItemEnabled(cutID, copyMenuItem);
+    popup->setItemEnabled(delID, delMenuItem);
+
+    popup->setItemEnabled(newID1,newMenuItem);
+    popup->setItemEnabled(newID2,newMenuItem);
+
+
+    if (inDeletedObjects) popup->changeItem(movID, tr("Undelete...") );
+
+    popup->exec( pos );
+}
+
+void ObjectManipulator::getMenuState(bool haveMoveTargets,
+                                     bool &dupMenuItem,
+                                     bool &moveMenuItem,
+                                     bool &copyMenuItem,
+                                     bool &pasteMenuItem,
+                                     bool &delMenuItem,
+                                     bool &newMenuItem,
+                                     bool &inDeletedObjects)
+{
+    dupMenuItem=true;
+    moveMenuItem=true;
+    copyMenuItem=true;
+    pasteMenuItem=true;
+    delMenuItem=true;
+    newMenuItem=true;
+
+    inDeletedObjects = false;
+
+    FWObject *delObjLib =
+        mw->db()->getById( FWObjectDatabase::getDeletedObjectsId());
 
     vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
         FWObject *obj= *i;
-        f1= (f1 && ! FWBTree::isSystem(obj) && ! Library::isA(obj) );
+
+        QString objPath = obj->getPath(true).c_str();
+
+        copyMenuItem  = copyMenuItem && FWBTree::getCopyMenuState(objPath);
+        pasteMenuItem = pasteMenuItem &&
+            FWBTree::getPasteMenuState(objPath) &&
+            (FWObjectClipboard::obj_clipboard->size()!=0);
+        delMenuItem   = delMenuItem && FWBTree::getDeleteMenuState(objPath);
+
+        if (pasteMenuItem)
+        {
+            /*
+             *  enable Paste menu item only if object can be pasted
+             */
+            vector<FWObject*>::iterator i;
+            for (i= FWObjectClipboard::obj_clipboard->begin();
+                 i!=FWObjectClipboard::obj_clipboard->end(); ++i)
+            {
+                FWObject *co= *i;
+                FWObject *nobj=pasteTo( obj , co , false, true);
+                pasteMenuItem = pasteMenuItem && (nobj!=NULL);
+            }
+        }
+
+        dupMenuItem=
+            (dupMenuItem && ! FWBTree::isSystem(obj) && ! Library::isA(obj) );
 
         inDeletedObjects = (delObjLib!=NULL && obj->isChildOf(delObjLib));
-        f1 = f1 && !inDeletedObjects;
+        dupMenuItem = dupMenuItem && !inDeletedObjects;
 
 // can't move system objects
-        f2 = f2 && ! FWBTree::isSystem(obj);
+        moveMenuItem = moveMenuItem && ! FWBTree::isSystem(obj);
 
 // can't move interfaces unless parent host object is also selected
         if ( Interface::isA(obj) &&
-     std::find(so.begin(),so.end(),obj->getParent())==so.end()) f2 = false;
+             std::find(so.begin(),so.end(),obj->getParent())==so.end())
+            moveMenuItem = false;
 
 // can't move ip addresses if parent is interface
-        if (IPv4::isA(obj) && Interface::isA(obj->getParent())) f2 = false;
+        if (IPv4::isA(obj) && Interface::isA(obj->getParent()))
+            moveMenuItem = false;
 
 // can't move physAddress objects
-        f2 = f2 && ! physAddress::isA(obj);
+        moveMenuItem = moveMenuItem && ! physAddress::isA(obj);
 
 // can't move read-only objects
-        f2 = f2 && ! obj->isReadOnly();
+        moveMenuItem = moveMenuItem && ! obj->isReadOnly();
 
 // can't move libraries unless in deleted objects
-        if (Library::isA(obj) && ! inDeletedObjects) f2 = false;
+        if (Library::isA(obj) && ! inDeletedObjects) moveMenuItem = false;
 
 // can't move if there is only one user-defined library in the tree
 // but we dont care about number of libraries if this will become
 // 'undelete' operation
-        if (moveTargets==0 && ! inDeletedObjects) f2 = false;
+        if (!haveMoveTargets && ! inDeletedObjects) moveMenuItem = false;
 
-        f3= (f3 && ! FWBTree::isSystem(currentObj) && ! Library::isA(currentObj));
-        f4= (f4 && ! FWBTree::isSystem(currentObj));
+//        copyMenuItem= (copyMenuItem &&
+//                       ! FWBTree::isSystem(currentObj) &&
+//                       ! Library::isA(currentObj));
+//        delMenuItem= (delMenuItem && ! FWBTree::isSystem(currentObj));
+
+        newMenuItem= (newMenuItem && ! obj->isReadOnly() );
+        Interface *intf = Interface::cast(obj);
+        if (intf && (intf->isDyn() || intf->isUnnumbered()))
+            newMenuItem = false;
 
-        f5= (f5 && ! obj->isReadOnly() );
     }
-
-    popup->setItemEnabled(dupID, f1); 
-    popup->setItemEnabled(movID, f2);
-    popup->setItemEnabled(copyID,f3);
-    
-    popup->setItemEnabled(cutID, f3);
-    popup->setItemEnabled(delID, f4);
-
-    popup->setItemEnabled(newID1,f5);
-    popup->setItemEnabled(newID2,f5);
-
-    if (inDeletedObjects) popup->changeItem(movID, tr("Undelete...") );
-
-    popup->exec( pos );
 }
 
 void ObjectManipulator::find()
@@ -1080,8 +1133,7 @@ void ObjectManipulator::duplicateObj(int libid)
     ObjectTreeView* ot=getCurrentObjectTree();
     ot->freezeSelection(true);
     FWObject *obj;
-    vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-    simplifySelection(so);
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
         obj= *i;
@@ -1108,8 +1160,7 @@ void ObjectManipulator::duplicateObjUnderSameParent()
     ObjectTreeView* ot=getCurrentObjectTree();
     ot->freezeSelection(true);
     FWObject *obj;
-    vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-    simplifySelection(so);
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
         obj= *i;
@@ -1243,8 +1294,7 @@ void ObjectManipulator::moveObj(int libid)
 
     FWObject *targetLib   = idxToLibs[libid];
 
-    vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-    simplifySelection(so);
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
         obj= *i;
@@ -1280,8 +1330,7 @@ void ObjectManipulator::copyObj()
     FWObject *obj;
     FWObjectClipboard::obj_clipboard->clear();
 
-    vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-    simplifySelection(so);
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
 
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
@@ -1317,7 +1366,8 @@ void ObjectManipulator::pasteObj()
     }
 }
 
-FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,bool openobj)
+FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,
+                                      bool openobj,bool validateOnly)
 {
     FWObject *ta=target;
     if (IPv4::isA(ta)) ta=ta->getParent();
@@ -1329,12 +1379,32 @@ FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,bool openob
         Host      *hst  = Host::cast(ta);   // works for firewall, too
         Interface *intf = Interface::cast(ta);
 
-        if (
-            (FWBTree::isSystem(ta) && FWBTree::validateForInsertion(ta,obj)) ||
+        if (FWBTree::isSystem(ta))
+        {
+            if (!FWBTree::validateForInsertion(ta,obj))
+            {
+                if (validateOnly) return NULL;
+
+                QMessageBox::warning(
+                    this,"Firewall Builder", 
+                    QObject::tr("Impossible to insert object %1 (type %2) into %3\nbecause of incompatible type.")
+                    .arg(obj->getName().c_str())
+                    .arg(obj->getTypeName().c_str())
+                    .arg(target->getName().c_str()),
+                    "&Continue", QString::null, QString::null,
+                    0, 1 );
+                
+                return obj;
+            }
+        }
+        
+        if ( FWBTree::isSystem(ta) ||
             (hst!=NULL  && hst->validateChild(obj)) ||
             (intf!=NULL && intf->validateChild(obj))
         )
         {
+            if (validateOnly) return obj;
+
 /* add a copy of the object to system group */
 
             FWObject *nobj=
@@ -1356,6 +1426,8 @@ FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,bool openob
 
         if (grp!=NULL && grp->validateChild(obj))
         {
+            if (validateOnly) return obj;
+
 /* check for duplicates. We just won't add an object if it is already there */
             string cp_id=obj->getId();
             list<FWObject*>::iterator j;
@@ -1375,34 +1447,25 @@ FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,bool openob
     }
     catch(FWException &ex)
     {
+        if (validateOnly) return NULL;
+
         QMessageBox::warning(
             this,"Firewall Builder", 
             ex.toString().c_str(),
             "&Continue", QString::null,QString::null,
             0, 1 );
-
     }
 
+    if (validateOnly) return NULL;
     return obj;
 }
 
-void ObjectManipulator::simplifySelection(vector<FWObject*> &selectedObjects)
-{
-    vector<FWObject*> so2 = selectedObjects;
-    for (vector<FWObject*>::iterator i=so2.begin();  i!=so2.end(); ++i)
-    {
-        for (vector<FWObject*>::iterator j=i;  j!=so2.end(); ++j)
-        {
-            vector<FWObject*>::iterator k=std::find(selectedObjects.begin(),
-                                                    selectedObjects.end(),*j);
-            if ( (*j)->isChildOf( *i ) && k!=selectedObjects.end())
-                selectedObjects.erase( k );
-        }
-    }
-}
-
 void ObjectManipulator::deleteObj()
 {
+    if (fwbdebug)
+        qDebug("ObjectManipulator::deleteObj selected %d objects ",
+               getCurrentObjectTree()->getNumSelected());
+
     if (getCurrentObjectTree()->getNumSelected()==0) return;
 
     FWObject *obj;
@@ -1413,8 +1476,7 @@ void ObjectManipulator::deleteObj()
     if (fwbdebug)
         qDebug("ObjectManipulator::deleteObj  delObjLib=%p",delObjLib);
 
-    vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-    simplifySelection(so);
+    vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
 
     for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
     {
@@ -1424,9 +1486,23 @@ void ObjectManipulator::deleteObj()
 
     emptyingTrashInLib = emptyingTrash && mw->editingLibrary();
 
+/* Ask user iff:
+ *
+ * we are emptying trash while editing library file (.fwl)
+ *    else
+ * 
+ * if we are not emptying Trash (i.e. not deleting "Deleted objects" library)
+ *    and
+ * (we delete more than one object
+ *    or
+ * we delete one object and it is not a library (because in this case
+ * we ask them later anyway))
+ */
+
     QString msg;
 
     if (emptyingTrashInLib)
+    {
         msg = tr(
     "Emptying of the 'Deleted Objects' in a library file is not recommended.\n"
     "When you remove deleted objects from a library file, Firewall Builder\n"
@@ -1435,18 +1511,25 @@ void ObjectManipulator::deleteObj()
     "unusual and unexpected behavior of the program.\n"
     "Do you want to delete selected objects anyway ?"
         );
-    else
-        msg = tr(
-    "When you delete an object, it is removed from the tree and\n"
-    "all groups and firewall policy rules that reference it.\n"
-    "Do you want to delete selected objects ?"
-        );
-
-    if ( (!emptyingTrash || emptyingTrashInLib) &&
-         QMessageBox::warning(
-             this,"Firewall Builder", msg,
-             "&Yes", "&No", QString::null,
-             0, 1 )!=0) return;
+        if (QMessageBox::warning(
+                this,"Firewall Builder", msg,
+                tr("&Yes"), tr("&No"), QString::null,
+                0, 1 )!=0) return;
+    } else
+    {
+        if (!emptyingTrash && (so.size()>1 || !Library::isA(so.front())))
+        {
+            msg = tr(
+                "When you delete an object, it is removed from the tree and\n"
+                "all groups and firewall policy rules that reference it.\n"
+                "Do you want to delete selected objects ?"
+            );
+            if (QMessageBox::warning(
+                    this,"Firewall Builder", msg,
+                    tr("&Yes"), tr("&No"), QString::null,
+                    0, 1 )!=0) return;
+        }
+    }
 
 
 /* need to work with a copy of the list of selected objects because
@@ -1466,32 +1549,38 @@ void ObjectManipulator::deleteObj()
         }
     }
 
-    for (vector<FWObject*>::iterator i=so2.begin();  i!=so2.end(); ++i)
+    try
     {
-        obj= *i;
+        for (vector<FWObject*>::iterator i=so2.begin();  i!=so2.end(); ++i)
+        {
+            obj= *i;
 
 //        openObject(obj,false);
 
-        if ( ! FWBTree::isSystem(obj) )
-        {
-            if (Library::isA(obj))
+            if ( ! FWBTree::isSystem(obj) )
             {
-                list<FWObject*> ll=mw->db()->getByType(Library::TYPENAME);
-                if (ll.size()==1)  return;
+                if (Library::isA(obj))
+                {
+                    list<FWObject*> ll=mw->db()->getByType(Library::TYPENAME);
+                    if (ll.size()==1)  return;
 
-                if (QMessageBox::warning(
-                        this,"Firewall Builder", 
-                        tr("When you delete a library, all objects that belong to it\ndisappear from the tree and all groups and rules that reference them.\nYou won't be able to reverse this operation later.\nDo you still want to delete library %1?")
-                        .arg(QString::fromUtf8(obj->getName().c_str())),
-                        "&Yes", "&No", QString::null,
-                        0, 1 )!=0 ) continue;
+                    if (QMessageBox::warning(
+                            this,"Firewall Builder", 
+                            tr("When you delete a library, all objects that belong to it\ndisappear from the tree and all groups and rules that reference them.\nYou won't be able to reverse this operation later.\nDo you still want to delete library %1?")
+                            .arg(QString::fromUtf8(obj->getName().c_str())),
+                            tr("&Yes"), tr("&No"), QString::null,
+                            0, 1 )!=0 ) continue;
+                }
+
+                if (oe->isVisible() && oe->getOpened()==obj) oe->hide();
+
+                delObj(obj);
             }
-
-            if (oe->isVisible() && oe->getOpened()==obj) oe->hide();
-
-            delObj(obj);
         }
     }
+    catch(FWException &ex)
+    {
+    }
 }
 
 void ObjectManipulator::delObj(FWObject *obj,bool openobj)
@@ -1507,10 +1596,14 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
         FWObject *parent=obj->getParent();
         FWObject *delObjLib = mw->db()->getById( DELETED_LIB );
 
+        if (fwbdebug)
+            qDebug("ObjectManipulator::delObj  deleted obj lib %p",
+                   delObjLib);
+
         bool islib  = Library::isA(obj);
 //        bool isintf = (Interface::isA(obj) && Firewall::isA(parent));
         bool isfw   = Firewall::isA(obj);
-        bool isDelObj = obj->isChildOf(delObjLib);
+        bool isDelObj = (delObjLib!=NULL && obj->isChildOf(delObjLib));
 
         if (fwbdebug)
             qDebug("ObjectManipulator::delObj  delete islib=%d isfw=%d isDelObj=%d",islib,isfw,isDelObj);
@@ -1541,7 +1634,7 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
         if (isfw) mw->deleteFirewall(obj);
 
 
-        removeObjectFromTreeView(obj);
+//        removeObjectFromTreeView(obj);
 
         if (islib && !isDelObj)
         {
@@ -1555,12 +1648,11 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
         } else
         {
             QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
-//            if (obj->isReadOnly()) obj->setReadOnly(false);
             mw->db()->recursivelyRemoveObjFromTree(obj);
             QApplication::restoreOverrideCursor();
         }
 
-//        if (isfw) mw->deleteFirewall(obj);
+        removeObjectFromTreeView(obj);
 
         if (!isDelObj)
         {
@@ -1581,7 +1673,7 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
             ex.toString().c_str(),
             "&Continue", QString::null,QString::null,
             0, 1 );
-
+        throw(ex);
     }
 }
 
@@ -1635,8 +1727,7 @@ void ObjectManipulator::groupObjects()
         ObjectTreeView* ot=getCurrentObjectTree();
         ot->freezeSelection(true);
 
-        vector<FWObject*> so = getCurrentObjectTree()->getSelectedObjects();
-        simplifySelection(so);
+        vector<FWObject*> so = getCurrentObjectTree()->getSimplifiedSelection();
 
         for (vector<FWObject*>::iterator i=so.begin();  i!=so.end(); ++i)
         {
@@ -1731,6 +1822,23 @@ void ObjectManipulator::selectionChanged()
 
     currentObj = obj;
 
+    bool dupMenuItem=true;
+    bool moveMenuItem=true;
+    bool copyMenuItem=true;
+    bool pasteMenuItem=true;
+    bool delMenuItem=true;
+    bool newMenuItem=true;
+    bool inDeletedObjects = false;
+
+    getMenuState(false,
+                 dupMenuItem,moveMenuItem,copyMenuItem,pasteMenuItem,
+                 delMenuItem,newMenuItem,inDeletedObjects);
+
+    mw->editCopyAction->setEnabled(copyMenuItem);
+    mw->editDeleteAction->setEnabled(delMenuItem);
+    mw->editCutAction->setEnabled(copyMenuItem);
+    mw->editPasteAction->setEnabled(pasteMenuItem);
+
     active=true;
     mw->unselectRules();
     info();
@@ -1863,7 +1971,7 @@ FWObject* ObjectManipulator::createObject(const QString &objType,
     {
         qDebug("lib: %s %s",lib->getName().c_str(), lib->getId().c_str());
         qDebug("lib: isReadOnly=%d isLoaded=%d",
-               lib->isReadOnly(), addOnLibs->isLoaded( lib->getName() ) );
+               lib->isReadOnly(), addOnLibs->isLoaded( lib->getName().c_str() ) );
         qDebug("libs->count()=%d", libs->count() );
     }
 
@@ -1888,7 +1996,7 @@ FWObject* ObjectManipulator::createObject(const QString &objType,
             qDebug("i=%d",i);
             qDebug("lib: %s %s",lib->getName().c_str(), lib->getId().c_str());
             qDebug("lib: isReadOnly=%d isLoaded=%d",
-                   lib->isReadOnly(), addOnLibs->isLoaded( lib->getName() ) );
+                   lib->isReadOnly(), addOnLibs->isLoaded( lib->getName().c_str() ) );
         }
         i++;
     }
@@ -1974,6 +2082,7 @@ void ObjectManipulator::newLibrary()
 void ObjectManipulator::newFirewall()
 {
     newFirewallDialog *nfd=new newFirewallDialog();
+    if (oe->isVisible()) oe->hide();
     nfd->exec();
     FWObject *o = nfd->getNewFirewall();
     delete nfd;
@@ -1991,6 +2100,7 @@ void ObjectManipulator::newFirewall()
 void ObjectManipulator::newHost()
 {
     newHostDialog *nhd=new newHostDialog();
+    if (oe->isVisible()) oe->hide();
     nhd->exec();
     FWObject *o = nhd->getNewHost();
     delete nhd;
@@ -2041,8 +2151,24 @@ void ObjectManipulator::newAddress()
     if ( currentObj->isReadOnly() ) return;
 
     FWObject *o;
+/*
+ * Oleg reports that his expectation was that "New Address" should
+ * always create an address object even if current selected object in
+ * the tree is an interface. I tend to agree with him, this was a
+ * usability issue because behavior of the program was different
+ * depending on which object was selected in the tree. I am changing
+ * it and will make it so "New Address" will always create a new
+ * Address object uner Objects/Addresses. Interface address can be
+ * created using context pop-up menu.
+ *                                         12/19/04 --vk
+ */
+    o=createObject(IPv4::TYPENAME,tr("New Address"));
+
+#if 0
     if (Interface::isA(currentObj))
     {
+        Interface *intf = Interface::cast(currentObj);
+        if (intf && (intf->isDyn() || intf->isUnnumbered())) return;
 	QString iname=QString("%1:%2:ip")
 	    .arg(QString::fromUtf8(currentObj->getParent()->getName().c_str()))
 	    .arg(QString::fromUtf8(currentObj->getName().c_str()));
@@ -2052,22 +2178,45 @@ void ObjectManipulator::newAddress()
     {
         o=createObject(IPv4::TYPENAME,tr("New Address"));
     }
+#endif
     openObject(o);
     edit(o);
 }
 
+void ObjectManipulator::newInterfaceAddress()
+{
+    if ( currentObj->isReadOnly() ) return;
+
+    if (Interface::isA(currentObj))
+    {
+        Interface *intf = Interface::cast(currentObj);
+        if (intf && (intf->isDyn() || intf->isUnnumbered())) return;
+	QString iname=QString("%1:%2:ip")
+	    .arg(QString::fromUtf8(currentObj->getParent()->getName().c_str()))
+	    .arg(QString::fromUtf8(currentObj->getName().c_str()));
+        FWObject *o=createObject(currentObj, IPv4::TYPENAME, iname);
+        openObject(o);
+        edit(o);
+    }
+}
+
 void ObjectManipulator::newPhysicalAddress()
 {
     if ( currentObj->isReadOnly() ) return;
 
     if (Interface::isA(currentObj))
     {
-	QString iname=QString("%1:%2:mac")
-	    .arg(QString::fromUtf8(currentObj->getParent()->getName().c_str()))
-	    .arg(QString::fromUtf8(currentObj->getName().c_str()));
-        FWObject *o=createObject(currentObj,physAddress::TYPENAME,iname);
-        openObject(o);
-        edit(o);
+        Interface *intf = Interface::cast(currentObj);
+        if (intf && (intf->isDyn() || intf->isUnnumbered())) return;
+        if (intf->getByType(physAddress::TYPENAME).empty())
+        {
+            QString iname=QString("%1:%2:mac")
+                .arg(QString::fromUtf8(currentObj->getParent()->getName().c_str()))
+                .arg(QString::fromUtf8(currentObj->getName().c_str()));
+            FWObject *o=createObject(currentObj,physAddress::TYPENAME,iname);
+            openObject(o);
+            edit(o);
+        }
     }
 }
 
@@ -2151,7 +2300,7 @@ bool ObjectManipulator::validateDialog()
 {
     if (currentObj==NULL) return true;
     if (!oe->isVisible()) return true;
-    return oe->validateDialog();
+    return oe->validateAndSave();
 }
 
 void ObjectManipulator::select()
