@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PrefsDialog.h,v 1.4 2004/06/23 06:30:11 vkurland Exp $
+  $Id: PrefsDialog.h,v 1.5 2004/12/17 07:23:19 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,16 +30,26 @@
 #include "config.h"
 #include <prefsdialog_q.h>
 
+#include "FWBSettings.h"
 #include "listOfLibraries.h"
 
 #include <list>
 #include <string>
+#include <map>
 
+#include <qstring.h>
+
+class QPushButton;
 
 class PrefsDialog : public prefsDialog_q
 {
     Q_OBJECT
 
+    void setButtonColor(QPushButton *btn,const QString &colorCode);
+    void changeColor(QPushButton *btn,FWBSettings::LabelColors colorCode);
+
+    std::map<int,QString> colors;
+    
  public:
     PrefsDialog(QWidget *parent);
     
@@ -51,6 +61,13 @@ public slots:
     virtual void findSSH();
     virtual void findSCP();
     virtual void libClick(QListViewItem* itm, const QPoint &p, int col);
+    virtual void changeRedColor();
+    virtual void changeOrangeColor();
+    virtual void changeYellowColor();
+    virtual void changeGreenColor();
+    virtual void changeBlueColor();
+    virtual void changePurpleColor();
+    virtual void changeGrayColor();
     
 };
 
