@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.cpp,v 1.49 2005/02/12 07:57:06 vkurland Exp $
+  $Id: RCS.cpp,v 1.57 2005/05/30 03:34:40 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -45,14 +45,19 @@
 
 #if defined(_WIN32)
 #  include <stdio.h>
-#  include <sys/timeb.h>
 #  include <sys/stat.h>
 #  include <fcntl.h>
 #  include <time.h>
+#  include <sys/timeb.h>
 #else
 #  include <unistd.h>
 #  include <string.h>
 #  include <errno.h>
+#  if defined(TM_IN_SYS_TIME)
+#    include <sys/time.h>
+#  else
+#    include <time.h>
+#  endif
 #endif
 
 #include <iostream>
@@ -137,22 +142,63 @@ bool Revision::operator!=(const Revision &r)
  ***********************************************************************/
 RCSEnvFix::RCSEnvFix()
 {
-#ifdef _WIN32
+    int tzoffset;
+    QString tzsign = "";
 
-/* need all this crap because Windows does not set environment
- * variable TZ by default, but rcs absolutely requires it
+    time_t clock; 
+    struct tm *ltm; 
+    time(&clock); 
+    ltm = (struct tm *) localtime(&clock);
+
+#if defined(HAVE_STRUCT_TM_TM_ZONE)
+/*
+ * struct tm has member tm_zone and should have member tm_gmtoff
+ * autoconf checks only for tm_zone, but these two member fields come
+ * together These fields are present in *BSD struct tm (including Mac
+ * OS X). Linux manual does not mention them, but they are present
+ * there as well. These files are missing in Windows version of time.h
+ *
+ * long tm_gmtoff is offset from UTC in seconds, it is negative in timezones
+ * west of GMT
  */
-    struct _timeb timebuffer;
 
-    _tzset();
-    _ftime( &timebuffer );
-    int tzoffset=timebuffer.timezone;
+    tzoffset = ltm->tm_gmtoff/60;
+    if (tzoffset<0)
+    {
+        tzoffset = -1*tzoffset;
+        tzsign = "-";
+    } else {
+        tzsign = "+";
+    }
+    
+#else
+// global variable timezone has seconds West of GMT (positive in
+// timezones west of GMT)
 
-    QString tz;
-    if (tzoffset<0) tz=QString("TZ=GMT+%1").arg(tzoffset/60);
-    else            tz=QString("TZ=GMT-%1").arg(tzoffset/60);
+    tzoffset = ((ltm->tm_isdst>0)?timezone-3600:timezone)/60;
+    if (tzoffset<0)
+    {
+        tzoffset = -1*tzoffset;
+        tzsign = "+";
+    } else {
+        tzsign = "-";
+    }
 
-    env.push_back(tz);
+#endif
+
+    TZOffset.sprintf("%02d:%02d",tzoffset/60,tzoffset%60);
+    TZOffset = tzsign + TZOffset;
+
+    if (fwbdebug)
+	qDebug("tzoffset: %d TZOffset: '%s'",tzoffset,TZOffset.ascii());
+
+#ifdef _WIN32
+/* need this crap because Windows does not set environment variable TZ
+ * by default, but rcs absolutely requires it. Even though I am using
+ * option "-z" with all RCS commands, stupid RCS on windows does not
+ * work if env var TZ is not set
+ */
+    env.push_back( QString("TZ=GMT")+TZOffset );
 
 /*
  * NB: need to prepend installation directory in front of PATH on
@@ -164,7 +210,6 @@ RCSEnvFix::RCSEnvFix()
  * directory as current dir, RCS tools fail without any error message.
  */
     env.push_back( QString("PATH=")+appRootDir+";"+getenv("PATH") );
-
 #endif
 
 /* also need to set env variable USER for rcs tools, but if the user name
@@ -304,7 +349,18 @@ RCS::~RCS()
     delete proc;
 }
 
-QStringList* RCS::getEnv() { return rcsenvfix->getEnv(); }
+QStringList* RCS::getEnv()
+{
+    if (rcsenvfix==NULL) rcsenvfix = new RCSEnvFix();
+    return rcsenvfix->getEnv();
+}
+
+RCSEnvFix*   RCS::getRCSEnvFix()
+{
+    if (rcsenvfix==NULL) rcsenvfix = new RCSEnvFix();
+    return rcsenvfix;
+}
+
 
 void RCS::readFromStdout()
 {
@@ -328,6 +384,7 @@ void RCS::abandon()
     proc->addArgument( co_file_name );
     proc->addArgument( "-q" );
     proc->addArgument( "-f" );
+    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
     proc->addArgument( QString("-u") );
     proc->addArgument( filename );
 
@@ -378,6 +435,7 @@ void RCS::add() throw(libfwbuilder::FWException)
     proc->addArgument( "-q" );
     proc->addArgument( "-i" );
     proc->addArgument( "-kb" );
+    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
     proc->addArgument( "-t-\"Initial checkin\"" );
     proc->addArgument( filename );
 
@@ -393,6 +451,7 @@ void RCS::add() throw(libfwbuilder::FWException)
             proc->addArgument( ci_file_name );
             proc->addArgument( "-q" );
             proc->addArgument( "-u" );
+            proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
             proc->addArgument( filename );
 
             stdoutBuffer="";
@@ -424,7 +483,7 @@ bool RCS::isInRCS()
     proc->clearArguments();
 
     proc->addArgument( rlog_file_name );
-    proc->addArgument( "-zLT" );
+    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
     proc->addArgument( "-R" );
     proc->addArgument( filename );
 
@@ -439,6 +498,11 @@ bool RCS::isInRCS()
     {
 /* exist status '1' means the file is not in RCS */
         inrcs=false;
+	if (fwbdebug)
+	{
+          QByteArray outp = proc->readStdout();
+          qDebug("Error running rlog: %s",outp.data());
+	}
         return false;
     }
     inrcs=true;
@@ -500,6 +564,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
         proc->addArgument( co_file_name );
         proc->addArgument( QString("-q") );
         proc->addArgument( QString("-kb") );
+        proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
         proc->addArgument( QString("-p")+rev );
         proc->addArgument( filename );
 
@@ -624,6 +689,7 @@ after the program crashed.").arg(locked_rev),
         proc->addArgument( "-q" );
         if (force)  proc->addArgument( "-f" );
         proc->addArgument( QString("-l")+rev );
+        proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
         proc->addArgument( filename );
 
         stdoutBuffer="";
@@ -677,6 +743,7 @@ bool RCS::ci( const QString &_lm,
     ciproc->addArgument( ci_file_name );
     if (unlock) ciproc->addArgument( "-u" );
     else        ciproc->addArgument( "-l" );
+    ciproc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
 
     ciproc->addArgument( filename );
 
@@ -769,14 +836,32 @@ void RCS::completeCI()
     if (loopLevel>1) QApplication::eventLoop()->exitLoop();
 }
 
+/**
+ * rlog - run rlog in the background and collect RCS log
+ *
+ * As it turns out, we can not trust rlog option "-zLT" to properly
+ * convert timezone information on Windows. This might be abug in the
+ * ported rlog. When timezone is east of GMT, ci properly converts
+ * when file is checked in, but rlog uses wrong sing and substracts
+ * offset instead of adding it. Suppose we are in Japan time zone
+ * (GMT+9), and file is checked in at 15:00 local time. Ci properly
+ * writes checkin time as 6:00 GMT, but rlog reports it as 21:00 on a
+ * previous day (it does -9 hours instead of +9 hours ). Option
+ * "-z+09:00" works properly
+ *
+ */
 QStringList RCS::rlog() throw(libfwbuilder::FWException)
 {
     proc->clearArguments();
 
     proc->addArgument( rlog_file_name );
-    proc->addArgument( "-zLT" );
+    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
+//    proc->addArgument( "-zLT" );
     proc->addArgument( filename );
 
+    if (fwbdebug)
+	qDebug("Running rlog:  %s",proc->arguments().join(" ").ascii());
+
     stdoutBuffer="";
     stderrBuffer="";
 
@@ -839,6 +924,7 @@ bool        RCS::isDiff(const QString &rev) throw(libfwbuilder::FWException)
     {
         if (selectedRev!="") proc->addArgument( QString("-r")+selectedRev );
     }
+    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
     proc->addArgument( filename );
 
     stdoutBuffer="";
