@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.h,v 1.18 2005/01/03 01:43:50 vkurland Exp $
+  $Id: RCS.h,v 1.20 2005/05/29 01:13:35 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -67,9 +67,11 @@ class Revision {
  */
 class RCSEnvFix {
     QStringList env;
+    QString     TZOffset;
  public:
     RCSEnvFix();
     QStringList* getEnv();
+    QString      getTZOffset() { return TZOffset; }
 };
 
 class RCS : public QObject {
@@ -211,6 +213,7 @@ class RCS : public QObject {
     void    setSelectedRev(const QString &rev) { selectedRev=rev; }
 
     static QStringList* getEnv();
+    static RCSEnvFix*   getRCSEnvFix();
 
  public slots:
 
