@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleOptionsDialog.cpp,v 1.7 2005/01/31 06:10:07 vkurland Exp $
+  $Id: RuleOptionsDialog.cpp,v 1.7.4.2 2005/09/13 04:55:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -73,36 +73,49 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     if (platform=="ipf")      wid=1;
     if (platform=="pf")       wid=2;
     if (platform=="ipfw")     wid=3;
-    if (platform=="pix")      wid=4;
+    if (platform=="pix" || platform=="fwsm")      wid=4;
 
     wStack->raiseWidget( wid );
     wStack->widget(wid)->show();
 
     QStringList  logLevels=getLogLevels( obj->getStr("platform").c_str() );
-
     ipt_logLevel->clear();
-    ipt_logLevel->insertStringList(logLevels);
-
+    ipt_logLevel->insertStringList(getScreenNames(logLevels));
     ipf_logLevel->clear();
-    ipf_logLevel->insertStringList(logLevels);
-
+    ipf_logLevel->insertStringList(getScreenNames(logLevels));
+    pix_logLevel->clear();
+    pix_logLevel->insertStringList(getScreenNames(logLevels));
 
     QStringList  logFacilities=getLogFacilities( obj->getStr("platform").c_str() );
-
     ipf_logFacility->clear();
-    ipf_logFacility->insertStringList(logFacilities);    
+    ipf_logFacility->insertStringList(getScreenNames(logFacilities));    
 
+    QStringList actionsOnReject=getActionsOnReject( obj->getStr("platform").c_str() );
+    ipt_actionOnReject->clear();
+    ipt_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
+    ipf_actionOnReject->clear();
+    ipf_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
+    pf_actionOnReject->clear();
+    pf_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
+    ipfw_actionOnReject->clear();
+    ipfw_actionOnReject->insertStringList(getScreenNames(actionsOnReject));
+
+    QStringList limitSuffixes=getLimitSuffixes( obj->getStr("platform").c_str() );
+    ipt_limitSuffix->clear();
+    ipt_limitSuffix->insertStringList(getScreenNames(limitSuffixes));
+    
+    
     data.clear();
-
+    
     if (platform=="iptables")
     {
         data.registerOption( ipt_logPrefix            , ropt,  "log_prefix" );
-        data.registerOption( ipt_logLevel             , ropt,  "log_level" );
+        data.registerOption( ipt_logLevel             , ropt,  "log_level", logLevels );
         data.registerOption( ipt_nlgroup              , ropt,  "ulog_nlgroup" );
         data.registerOption( ipt_rule_name_accounting , ropt,  "rule_name_accounting" );
-        data.registerOption( ipt_actionOnReject       , ropt,  "action_on_reject");
+        data.registerOption( ipt_actionOnReject       , ropt,  "action_on_reject" ,actionsOnReject);
         data.registerOption( ipt_limit                , ropt,  "limit_value" );
-        data.registerOption( ipt_limitSuffix          , ropt,  "limit_suffix" );
+        data.registerOption( ipt_limitSuffix          , ropt,  "limit_suffix", limitSuffixes);
         data.registerOption( ipt_burst                , ropt,  "limit_burst" );
         data.registerOption( ipt_assumeFwIsPartOfAny  , ropt,  "firewall_is_part_of_any_and_networks" );
         data.registerOption( ipt_stateless            , ropt,  "stateless" );
@@ -116,9 +129,9 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
 
     if (platform=="ipf")
     {
-        data.registerOption( ipf_logFacility     , ropt,  "ipf_log_facility");
-        data.registerOption( ipf_logLevel        , ropt,  "log_level" );
-        data.registerOption( ipf_actionOnReject  , ropt,  "action_on_reject");
+        data.registerOption( ipf_logFacility     , ropt,  "ipf_log_facility", logFacilities);
+        data.registerOption( ipf_logLevel        , ropt,  "log_level" , logLevels);
+        data.registerOption( ipf_actionOnReject  , ropt,  "action_on_reject", actionsOnReject);
         data.registerOption( ipf_masq_icmp       , ropt,  "ipf_return_icmp_as_dest");
         data.registerOption( ipf_stateless       , ropt,  "stateless" );
         data.registerOption( ipf_keep_frags      , ropt,  "ipf_keep_frags" );
@@ -127,7 +140,7 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
     if (platform=="pf")
     {
         data.registerOption( pf_logPrefix        , ropt,  "log_prefix" );
-        data.registerOption( pf_actionOnReject   , ropt,  "action_on_reject");
+        data.registerOption( pf_actionOnReject   , ropt,  "action_on_reject",actionsOnReject);
         data.registerOption( pf_stateless        , ropt,  "stateless" );
         data.registerOption( pf_rule_max_state   , ropt,  "pf_rule_max_state" );
         data.registerOption( pf_source_tracking  , ropt,  "pf_source_tracking" );
@@ -137,14 +150,14 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
 
     if (platform=="ipfw")
     {
-        data.registerOption( ipfw_actionOnReject , ropt,"action_on_reject");
+        data.registerOption( ipfw_actionOnReject , ropt,"action_on_reject",actionsOnReject);
         data.registerOption( ipfw_stateless      , ropt,"stateless" );
     }
 
-    if (platform=="pix")
+    if (platform=="pix" || platform=="fwsm")
     {
         string vers="version_"+p->getStr("version");
-        if ( Resources::platform_res["pix"]->getResourceBool(
+        if ( Resources::platform_res[platform]->getResourceBool(
               "/FWBuilderResources/Target/options/"+vers+"/pix_rule_syslog_settings"))
         {
             pix_disable_rule_log->setEnabled(true);
@@ -152,7 +165,7 @@ void RuleOptionsDialog::loadFWObject(FWObject *o)
             pix_log_interval->setEnabled(true);
 
             data.registerOption( pix_disable_rule_log, ropt,"disable_logging_for_this_rule" );
-            data.registerOption( pix_logLevel        , ropt,"log_level" );
+            data.registerOption( pix_logLevel        , ropt,"log_level" ,logLevels);
             data.registerOption( pix_log_interval    , ropt,"log_interval" );
         } else
         {
