@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleSetView.cpp,v 1.94 2005/02/08 06:19:51 vkurland Exp $
+  $Id: RuleSetView.cpp,v 1.95.4.1 2005/09/04 20:49:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -1771,41 +1771,67 @@ void RuleSetView::contentsDragEnterEvent( QDragEnterEvent *ev)
 
 void RuleSetView::dragMoveEvent( QDragMoveEvent *ev)
 {
-    if (FWObjectDrag::canDecode(ev) && !ruleset->isReadOnly())
+    QWidget *fromWidget = ev->source();
+    // The source of DnD object must be the same instance of fwbuilder
+    if (fromWidget)
     {
-        QHeader *hh = horizontalHeader();
-        QHeader *vh = verticalHeader();
-    
-        int  row = rowAt( ev->pos().y() + contentsY() - hh->height() );
-        int  col = columnAt( ev->pos().x() + contentsX() - vh->width() );
-
-        if (col<0 || ( getColType(col)!=Object && getColType(col)!=Time) )
+        if (FWObjectDrag::canDecode(ev) && !ruleset->isReadOnly())
         {
-            ev->acceptAction(false);
-            return;
-        }
+            QHeader *hh = horizontalHeader();
+            QHeader *vh = verticalHeader();
+        
+            int  row = rowAt( ev->pos().y() + contentsY() - hh->height() );
+            int  col = columnAt( ev->pos().x() + contentsX() - vh->width() );
 
-        RuleElement *re = getRE(row,col);
-        if (re==NULL)
-        {
-            ev->acceptAction(false);
-            return;
-        }
-
-        bool  acceptE = true;
-        list<FWObject*> dragol;
-        if (FWObjectDrag::decode(ev, dragol))
-        {
-            for (list<FWObject*>::iterator i=dragol.begin();
-                 i!=dragol.end(); ++i)
+            if (col<0 || ( getColType(col)!=Object && getColType(col)!=Time) )
             {
-                FWObject *dragobj = *i;
-                assert(dragobj!=NULL);
-                
-                acceptE &= re->validateChild(dragobj);
+                ev->acceptAction(false);
+                return;
+            }
+
+            RuleElement *re = getRE(row,col);
+            if (re==NULL)
+            {
+                ev->acceptAction(false);
+                return;
+            }
+
+            bool  acceptE = true;
+            list<FWObject*> dragol;
+
+            /*
+             * See bug 1226069  Segfault: Drag&Drop between two instances
+             *
+             * There is a problem with this code: Since we pass "live" pointer to
+             * FWObject* object in the drag event, drag&drop should not be used to
+             * pass objects between different instances of the GUI running at the
+             * same time. Trying to do so causes receiving program to crash
+             * because it tries to access an object using memory pointer that it
+             * obtained from another program. Apparently C++ standard does not
+             * define behavior of the program in case it tries to access memory
+             * using bad pointer, so it does not throw any exception we could
+             * catch. Real solution, of course, is to pass object's ID together
+             * with some identification for the data file in the drag event and
+             * then on receiving end scan the tree looking for this
+             * object. Unfortunately this is too slow with our current API desing.
+             *
+             * TODO: The problem requires more detailed investigation, possibly
+             * including API redesign.
+             */
+
+            if (FWObjectDrag::decode(ev, dragol))
+            {
+                for (list<FWObject*>::iterator i=dragol.begin();
+                     i!=dragol.end(); ++i)
+                {
+                    FWObject *dragobj = NULL;
+                    dragobj = dynamic_cast<FWObject*>(*i);
+                    if(dragobj!=NULL)
+                        acceptE &= re->validateChild(dragobj);
+                }
+                ev->acceptAction( acceptE );
+                return;
             }
-            ev->acceptAction( acceptE );
-            return;
         }
     }
     ev->accept(false);
