@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHPIX.cpp,v 1.10 2004/11/24 06:46:14 vkurland Exp $
+  $Id: SSHPIX.cpp,v 1.11.4.2 2005/09/12 06:12:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,10 +53,10 @@ SSHPIX::SSHPIX(const QString &_h,
                const QString &_ep,
                const QStringList &_in) : SSHSession(_h,args,_p,_ep,_in)
 {
-    normal_prompt="> ";
+    normal_prompt="> $";
     fwb_prompt="--**--**--";
-    enable_prompt="# ";
-    pwd_prompt="'s password: ";
+    enable_prompt="# $|# *Access Rules Download Complete";
+    pwd_prompt="'s password: $";
     epwd_prompt="Password: ";
     scp_pwd_prompt="Password: ";
     ssh_pwd_prompt="'s password: ";
@@ -121,7 +121,7 @@ void SSHPIX::stateMachine()
     {
         if ( stdoutBuffer.findRev(*i,-1)!=-1 )
         {
-            emit printStdout_sign( tr("*** Fatal error :") );
+            emit printStdout_sign( tr("\n*** Fatal error :") );
             emit printStdout_sign( stdoutBuffer+"\n" );
             stdoutBuffer="";
             terminate();
@@ -134,7 +134,7 @@ void SSHPIX::stateMachine()
     {
     case NONE:
     {
-        if ( cmpPrompt(stdoutBuffer,pwd_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(pwd_prompt)) )
         {
             stdoutBuffer="";
             proc->writeToStdin( pwd );
@@ -144,7 +144,7 @@ void SSHPIX::stateMachine()
 /* we may get to LOGGEDIN state directly from NONE, for example when
  * password is supplied on command line to plink.exe
  */
-        if (cmpPrompt(stdoutBuffer,normal_prompt) )
+        if (cmpPrompt(stdoutBuffer,QRegExp(normal_prompt)) )
         {
             state=LOGGEDIN;
             emit printStdout_sign( "\n");
@@ -186,7 +186,7 @@ void SSHPIX::stateMachine()
     break;
 
     case LOGGEDIN:
-        if ( cmpPrompt(stdoutBuffer,epwd_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(epwd_prompt)) )
         {
             stdoutBuffer="";
             proc->writeToStdin( epwd );
@@ -196,7 +196,7 @@ void SSHPIX::stateMachine()
         break;
 
     case WAITING_FOR_ENABLE:
-        if ( cmpPrompt(stdoutBuffer,epwd_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(epwd_prompt)) )
         {
             stdoutBuffer="";
             proc->writeToStdin( epwd );
@@ -204,7 +204,7 @@ void SSHPIX::stateMachine()
             state=WAITING_FOR_ENABLE;
             break;
         }
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {       
             emit printStdout_sign( tr("In enable mode."));
             emit printStdout_sign( "\n");
@@ -212,7 +212,7 @@ void SSHPIX::stateMachine()
         }
 
     case ENABLE:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             stdoutBuffer="";
 
@@ -236,7 +236,7 @@ void SSHPIX::stateMachine()
         break;
 
     case EXECUTING_COMMAND:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {       
             QApplication::eventLoop()->exitLoop();
             state=COMMAND_DONE;
@@ -244,7 +244,7 @@ void SSHPIX::stateMachine()
         break;
 
     case WAITING_FOR_CONFIG_PROMPT:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             state=CONFIG;
 
@@ -309,7 +309,7 @@ void SSHPIX::stateMachine()
 #endif
 
     case PUSHING_CONFIG:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
         loop1:
             if ( allConfig.size()!=0 )
@@ -363,14 +363,14 @@ void SSHPIX::stateMachine()
         break;
 
     case GET_ACLS:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             QTimer::singleShot( 0, this, SLOT(getACLs()) );
         }
         break;
 
     case GET_OG:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             QTimer::singleShot( 0, this, SLOT(getObjectGroups()) );
         }
@@ -378,14 +378,14 @@ void SSHPIX::stateMachine()
 
 
     case CLEAR_ACLS:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             QTimer::singleShot( 0, this, SLOT(clearACLs()) );
         }
         break;
 
     case CLEAR_OG:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             QTimer::singleShot( 0, this, SLOT(clearObjectGroups()) );
         }
@@ -393,7 +393,7 @@ void SSHPIX::stateMachine()
 
 
     case EXIT_FROM_CONFIG:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
 /*
  * dry_run means run through the commands without executing them on
@@ -416,7 +416,7 @@ void SSHPIX::stateMachine()
         break;
 
     case SAVE_CONFIG:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             if (saveStandby && ncmd!=0)
             {
@@ -433,7 +433,7 @@ void SSHPIX::stateMachine()
         break;
 
     case SAVE_STANDBY:
-        if ( cmpPrompt(stdoutBuffer,enable_prompt) )
+        if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
         {
             emit printStdout_sign( tr("Exiting\n") );
             state=EXIT;
