@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.cpp,v 1.10 2005/02/04 07:19:55 vkurland Exp $
+  $Id: SSHSession.cpp,v 1.14.4.5 2005/09/08 06:05:57 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -70,8 +70,11 @@ SSHSession::SSHSession(const QString &_h,
     verbose = false;
     closeStdin = false;
     error = false;
+    endOfCopy = false;
 
     proc = NULL;
+    heartBeatTimer = new QTimer(this);
+    connect(heartBeatTimer, SIGNAL(timeout()), this, SLOT(heartBeat()) );
 
     newKeyMsg = tr("You are connecting to the firewall <b>'%1'</b> for the first time. It has provided you its identification in a form of its host public key. The fingerprint of the host public key is: \"%2\" You can save the host key to the local database by pressing YES, or you can cancel connection by pressing NO. You should press YES only if you are sure you are really connected to the firewall <b>'%3'</b>.");
 
@@ -99,6 +102,12 @@ void SSHSession::startSession()
     proc->setCommunication(
         QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
+    heartBeatTimer->start(1000);
+
+    if (fwbdebug)
+        qDebug("SSHSession::startSession  this=%p  proc=%p   heartBeatTimer=%p",
+               this,proc,heartBeatTimer);
+
     connect(proc,SIGNAL(readyReadStdout()), this,  SLOT(readFromStdout() ) );
     connect(proc,SIGNAL(readyReadStderr()), this,  SLOT(readFromStderr() ) );
     connect(proc,SIGNAL(processExited()),   this,  SLOT(processExited() ) );
@@ -181,6 +190,12 @@ void SSHSession::setOptions(instConf *cnf)
 
 void SSHSession::terminate()
 {
+    if (fwbdebug)
+        qDebug("SSHSession::terminate     this=%p  proc=%p   heartBeatTimer=%p",
+               this,proc,heartBeatTimer);
+
+    heartBeatTimer->stop();
+
     if (proc!=NULL)
     {
         disconnect(proc,SIGNAL(readyReadStdout()),
@@ -194,6 +209,13 @@ void SSHSession::terminate()
             qDebug("SSHSession::terminate   terminating child process pid=%ld",
                    proc->processIdentifier());
 
+        QString s=QString(proc->readStdout());
+        if (!quiet)
+        {
+            s.replace('\r',"");    
+            emit printStdout_sign(s);
+        }
+
         proc->kill();
         delete proc;
         proc=NULL;
@@ -205,39 +227,110 @@ void SSHSession::stateMachine()
 {
 }
 
+/*
+ * signal wroteToStdin is connected to slot readyToSend. Can not send
+ * next line in this slot because on win32 it emits the signal and
+ * thus calls the same slot recursively, without exiting first. On
+ * Linux and Mac it seems to exit and then emit the signal and call
+ * slot on the next pass of the even loop. Since on win32 this does
+ * not happen, need to schedule sending next line via single shot
+ * timer instead of calling it directly.
+ */
+void SSHSession::readyToSend()
+{
+    QTimer::singleShot( 0, this, SLOT(sendLine()) );
+}
+
+void SSHSession::sendLine()
+{
+    int n=0;
+    while (input.size()!=0 && n<10)
+    {
+        QString s = input.front();
+        if (fwbdebug)
+            qDebug("SSHSession::sendLine : %d lines to go -- %s",input.size(),s.ascii());
+
+        input.pop_front();
+
+        stdoutBuffer="";
+        proc->writeToStdin( s+"\n"); // send even if s is empty
+        n++;
+    }
+    emit updateProgressBar_sign(input.size(),false);
+        
+    if (input.size()==0)
+    {
+        if (fwbdebug) qDebug("SSHUnx::sendLine - entire file sent, closeStdin=%d",
+                             closeStdin);
+        endOfCopy = true;
+    }
+}
+
 void SSHSession::allDataSent()
 {
     if (fwbdebug)
         qDebug("SSHSession::allDataSent   closing stdin");
 
-    disconnect(proc,SIGNAL(wroteToStdin()),this,SLOT(allDataSent()));
+    disconnect(proc,SIGNAL(wroteToStdin()),this,SLOT(readyToSend()));
+
 #ifdef _WIN32
     Sleep(2000);
 #endif
     proc->closeStdin();
+#ifdef _WIN32
+    Sleep(1000);
+#endif
+    readFromStdout();
+}
+
+void SSHSession::heartBeat()
+{
+    if (fwbdebug) qDebug("SSHSession::heartBeat");
+    readFromStderr();
+    readFromStdout();
+    if (endOfCopy && closeStdin) 
+    {
+        allDataSent();
+        endOfCopy = false;
+    }
 }
 
 void SSHSession::readFromStdout()
 {
-    QString s=QString(proc->readStdout());
-    stdoutBuffer=stdoutBuffer + s;
-
-//    if (verbose && !quiet)
-    if (!quiet)
+    if (proc)
     {
-        s.replace('\r',"");    
-        emit printStdout_sign(s);
-    }
+        QByteArray ba = proc->readStdout();
+        if (ba.size()!=0)
+        {
+            if (fwbdebug)
+                qDebug("SSHSession::readFromStdout read %d bytes from child process",ba.size());
+            QString s=QString(ba);
+            stdoutBuffer=stdoutBuffer + s;
+
+            if (!quiet)
+            {
+                s.replace('\r',"");    
+                emit printStdout_sign(s);
+            }
 
 /* state machine operates on stdoutBuffer directly */
-    stateMachine();
+            stateMachine();
+        }
+    }
 }
 
 void SSHSession::readFromStderr()
 {
-    QString s=QString(proc->readStderr());
-    emit printStdout_sign(s);
-    stderrBuffer=stderrBuffer + QString(s);
+    if (proc)
+    {
+        QByteArray ba = proc->readStderr();
+        if (ba.size()!=0)
+        {
+            QString s=QString(ba);
+            emit printStdout_sign(s);
+            stderrBuffer=stderrBuffer + QString(s);
+        }
+    }
 }
 
 void SSHSession::processExited()
@@ -245,25 +338,45 @@ void SSHSession::processExited()
     int retcode=proc->exitStatus();
     QString exitStatus = (retcode)?QObject::tr("ERROR"):QObject::tr("OK");
     
-    emit printStdout_sign(tr("SSH session terminated, exit status: %1").arg(exitStatus));
+    emit printStdout_sign(tr("SSH session terminated, exit status: %1").arg(retcode));
     if (retcode) error=true;
     emit sessionFinished_sign();
 }
 
 bool SSHSession::cmpPrompt(const QString &str,const QString &prompt)
 {
+    if (fwbdebug)
+        qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s'",
+               str.ascii(),prompt.ascii());
+
     if (str.isEmpty()) return false;
 
-    bool res=false;
-    if (str.length()>=prompt.length())
+    bool res=(str.findRev(prompt,-1)!=-1);
+    if (!res)
     {
-        res=(unsigned(str.findRev(prompt,-1))==str.length()-prompt.length());
-        if (!res)
-        {
-            QString s=str.stripWhiteSpace();
-            res=(unsigned(s.findRev(prompt,-1))==s.length()-prompt.length());
-        }
+        QString s=str.stripWhiteSpace();
+        res=(s.findRev(prompt,-1)!=-1);
     }
+
+    if (fwbdebug)
+        qDebug("SSHSession::cmpPrompt: res=%d",res);
+
+    return res;
+}
+
+bool SSHSession::cmpPrompt(const QString &str,const QRegExp &prompt)
+{
+    if (fwbdebug)
+        qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s' (regexp)",
+               str.ascii(),prompt.pattern().ascii());
+
+    if (str.isEmpty()) return false;
+
+    bool res=(str.findRev(prompt,-1)!=-1);
+
+    if (fwbdebug)
+        qDebug("SSHSession::cmpPrompt: res=%d",res);
+
     return res;
 }
 
