@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.h,v 1.7 2004/11/24 06:46:14 vkurland Exp $
+  $Id: SSHSession.h,v 1.9.4.3 2005/09/08 06:05:57 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,12 +32,13 @@
 
 #include <qstring.h>
 #include <qstringlist.h>
+#include <qregexp.h>
 #include <qprocess.h>
 #include <qobject.h>
 
 #include <fstream>
 
-
+class QTimer;
 class instConf;
 
 class SSHSession : public QObject {
@@ -47,6 +48,7 @@ class SSHSession : public QObject {
  protected:
 
     QProcess   *proc;
+    QTimer     *heartBeatTimer;
     QString     stdoutBuffer;
     QString     stderrBuffer;
     QString     ssh;
@@ -59,6 +61,7 @@ class SSHSession : public QObject {
     bool       logged_in;
     bool       enable;
     bool       configure;
+    bool       endOfCopy;
     
     enum State { NONE,
                  LOGGEDIN,
@@ -135,6 +138,7 @@ class SSHSession : public QObject {
 
     
     bool cmpPrompt(const QString &str,const QString &prompt);
+    bool cmpPrompt(const QString &str,const QRegExp &prompt);
     
 public:
     
@@ -175,7 +179,10 @@ public slots:
      void readFromStdout();
      void readFromStderr();
      void processExited();
+     void readyToSend();
+     void sendLine();
      void allDataSent();
+     void heartBeat();
      
  signals:
 
