@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHUnx.cpp,v 1.8 2004/11/14 01:32:27 vkurland Exp $
+  $Id: SSHUnx.cpp,v 1.12.4.3 2005/09/01 06:07:28 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -89,7 +89,7 @@ void SSHUnx::stateMachine()
     {
         if ( stdoutBuffer.findRev(*i,-1)!=-1 )
         {
-            emit printStdout_sign( tr("*** Fatal error :") );
+            emit printStdout_sign( tr("\n*** Fatal error :") );
             emit printStdout_sign( stdoutBuffer+"\n" );
             stdoutBuffer="";
             terminate();
@@ -123,6 +123,8 @@ void SSHUnx::stateMachine()
         {
             state=PUSHING_CONFIG;
             if (!quiet) emit printStdout_sign( tr("Logged in\n") );
+            if (fwbdebug)
+                qDebug("SSHUnx::stateMachine logged in");
 //            proc->writeToStdin( "\n" );
 //            stdoutBuffer="";
             goto push_files;
@@ -185,27 +187,11 @@ void SSHUnx::stateMachine()
 
         if (input.size()!=0)
         {
-            if (fwbdebug)
-                qDebug("SSHUnx::stateMachine() - sending a file");
-
-            while (input.size()!=0)
-            {
-                QString s = input.front();
-                input.pop_front();
-
-                stdoutBuffer="";
-                proc->writeToStdin( s+"\n"); // send even if s is empty
-            } 
-
-            if (fwbdebug)
-                qDebug("SSHUnx::stateMachine() - entire file sent, closeStdin=%d",
-                       closeStdin);
-
-            if (closeStdin)
-            {
-                connect(proc,SIGNAL(wroteToStdin()),this,SLOT(allDataSent()));
-                proc->writeToStdin( "\n");
-            }
+            if (fwbdebug) qDebug("SSHUnx::stateMachine - sending a file");
+            emit updateProgressBar_sign(input.size(),true);
+            connect(proc,SIGNAL(wroteToStdin()),this,SLOT(readyToSend()));
+            sendLine();
+            break;
         }
         break;
 
