@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: filePropDialog.h,v 1.1 2004/05/24 05:51:10 vkurland Exp $
+  $Id: filePropDialog.h,v 1.2 2005/05/28 21:16:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,10 +27,14 @@
 #ifndef __FILEPROPDIALOG_H_
 #define __FILEPROPDIALOG_H_
 
+#include <qstring.h>
+#include <qstringlist.h>
+
 #include "config.h"
 #include <filepropdialog_q.h>
 
 class RCS;
+class QPrinter;
 
 class filePropDialog : public filePropDialog_q
 {
@@ -38,7 +42,12 @@ class filePropDialog : public filePropDialog_q
     
  public:
     filePropDialog(QWidget *parent, RCS *rcs);
+    void setPrinter(QPrinter *printer);
     
+ private: 
+    QPrinter *printer;
+ public slots:
+    virtual void printRevHistory( );
 };
 
 #endif // __FILEPROPDIALOG_H
