@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.cpp,v 1.60 2005/04/10 07:16:39 vkurland Exp $
+  $Id: instDialog.cpp,v 1.65.4.4 2005/09/13 04:55:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -115,7 +115,7 @@ instDialog::instDialog(instConf *_c)
         Resources::getTargetOptionStr(cnf->fwobj->getStr("host_OS"),
                                       "activation/timeout_units").c_str();
 
-    if (platform!="pix")
+    if (platform!="pix" && platform!="fwsm")
     {
         incr->hide();
         test->hide();
@@ -144,7 +144,7 @@ instDialog::instDialog(instConf *_c)
     stripComments->hide();
     compressScript->hide();
 
-    if (platform=="pix")
+    if (platform=="pix" || platform=="fwsm")
     {
         progressBar->show();
         stripComments->show();
@@ -442,7 +442,7 @@ void instDialog::selected(const QString &title)
 
         QStringList  args;
 
-        if (cnf->fwobj->getStr("platform")=="pix")
+        if (cnf->fwobj->getStr("platform")=="pix" || cnf->fwobj->getStr("platform")=="fwsm")
         {
 #ifdef _WIN32
             args.push_back(ssh);
@@ -464,6 +464,7 @@ void instDialog::selected(const QString &title)
 #else
             args.push_back(argv0.c_str());
             args.push_back("-X");   // fwbuilder works as ssh wrapper
+//            args.push_back("-d");
             args.push_back("-t");
             args.push_back("-t");
 #endif
@@ -496,24 +497,17 @@ void instDialog::selected(const QString &title)
  */
             QString s;
 
-            if (cnf->testRun)
-            {
-                s=Resources::getTargetOptionStr(
-                    cnf->fwobj->getStr("host_OS"),
-                    "activation/fwdir_test").c_str();
-            } else
-            {
 /* user_can_change_install_dir */
-                bool uccid=Resources::getTargetOptionBool(
-                    cnf->fwobj->getStr("host_OS"),"user_can_change_install_dir");
+            bool uccid=Resources::getTargetOptionBool(
+                cnf->fwobj->getStr("host_OS"),"user_can_change_install_dir");
 
-                if (uccid)
-                    s=cnf->fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
+            if (uccid)
+                s=cnf->fwobj->getOptionsObject()->getStr("firewall_dir").c_str();
+
+            if (s.isEmpty()) s=Resources::getTargetOptionStr(
+                cnf->fwobj->getStr("host_OS"),
+                "activation/fwdir").c_str();
 
-                if (s.isEmpty()) s=Resources::getTargetOptionStr(
-                    cnf->fwobj->getStr("host_OS"),
-                    "activation/fwdir").c_str();
-            }
             cnf->fwdir     = s;
 
             continueRun();
@@ -580,6 +574,9 @@ void instDialog::initiateCopy(const QString &file)
     if (fwbdebug)
         qDebug("instDialog::initiateCopy for the file %s",file.ascii());
 
+    QString platform=cnf->fwobj->getStr("platform").c_str();
+    if (platform!="pix" && platform!="fwsm") progressBar->show();
+
     QString ff = cnf->wdir+"/"+file;
 
     std::ifstream   *config_file;
@@ -616,6 +613,7 @@ void instDialog::initiateCopy(const QString &file)
 #else
     args.push_back(argv0.c_str());
     args.push_back("-X");   // fwbuilder works as ssh wrapper
+//    if (fwbdebug>1) args.push_back("-d");
 //    args.push_back("-t");
 //    args.push_back("-t");
 #endif
@@ -625,6 +623,7 @@ void instDialog::initiateCopy(const QString &file)
     if (!cnf->sshArgs.isEmpty())
         args += QStringList::split(" ",cnf->sshArgs);
 
+    if (cnf->verbose) args.push_back("-v");
     if (!cnf->user.isEmpty())
     {
         args.push_back("-l");
@@ -638,6 +637,9 @@ void instDialog::initiateCopy(const QString &file)
     if (cnf->user=="root")   optpath += "root/";
     else                     optpath += "reg_user/";
 
+    if (cnf->testRun)        optpath += "test/";
+    else                     optpath += "run/";
+
     optpath+="copy";
 
     QString cmd=Resources::getTargetOptionStr(cnf->fwobj->getStr("host_OS"),
@@ -654,8 +656,9 @@ void instDialog::initiateCopy(const QString &file)
 
     args.push_back(cmd);
         
-    appendRich( tr("\nCopying %1 -> %2\n")
-                .arg(cnf->wdir+"/"+file).arg(cnf->fwdir) );
+    appendRich( tr("\nCopying %1 -> %2:%3\n")
+                .arg(cnf->wdir+"/"+file).arg(cnf->maddr).arg(cnf->fwdir) );
+    appendRich( "\n");
 
     if (cnf->verbose) displayCommand(args);
 
@@ -711,12 +714,13 @@ void instDialog::continueRun()
         return;
     }
 
-    if (cnf->fwobj->getStr("platform")=="pix")
+    if (cnf->fwobj->getStr("platform")=="pix" || cnf->fwobj->getStr("platform")=="fwsm")
     {
         setFinishEnabled( page(1), true );
         return;
     } else
     {
+
         if (!confFiles.empty())
         {
             QString cnffile = confFiles.front();
@@ -726,6 +730,8 @@ void instDialog::continueRun()
         {
             QStringList args;
 
+            progressBar->hide();
+
 #ifdef _WIN32
             args.push_back(ssh);
 
@@ -763,6 +769,7 @@ void instDialog::continueRun()
             args.push_back( cmd );
 
             appendRich( tr("Activating new policy\n") );
+            appendRich( "\n");
 
             if (cnf->verbose) displayCommand(args);
 
@@ -780,6 +787,8 @@ void instDialog::continueRun()
 
 void instDialog::runSSH(SSHSession *s)
 {
+    if (fwbdebug) qDebug("instDialog::runSSH()");
+
     session = s;
 
     session->setOptions(cnf);
@@ -814,12 +823,13 @@ void instDialog::processExited()
 }
 
 /* user clicked 'Cancel' */
-void instDialog::rejected()
+void instDialog::reject()
 {
+    if (fwbdebug) qDebug("instDialog::reject()");
     if (session!=NULL)
     {
         if (fwbdebug)
-            qDebug("instDialog::rejected()  killing ssh session");
+            qDebug("instDialog::reject()  killing ssh session");
 
         disconnect(session,SIGNAL(printStdout_sign(const QString&)),
                    this,SLOT(append(const QString&)));
@@ -835,6 +845,7 @@ void instDialog::rejected()
         delete session;
         session=NULL;
     }
+    QWizard::reject();
 }
 
 
@@ -853,7 +864,7 @@ void instDialog::hideEvent( QHideEvent *ev)
 void instDialog::testRunRequested()
 {
     bool f = testRun->isChecked();
-    if (cnf->fwobj->getStr("host_OS")=="pix_os")
+    if (cnf->fwobj->getStr("host_OS")=="pix_os" || cnf->fwobj->getStr("host_OS")=="fwsm_os")
     {
         rollback->setChecked( false );
         f=false;
