@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.h,v 1.19 2005/02/04 07:19:56 vkurland Exp $
+  $Id: instDialog.h,v 1.20 2005/07/03 05:08:04 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -92,7 +92,7 @@ class instDialog : public instDialog_q {
  protected slots:
      void processExited();
      void selected(const QString &title);
-     void rejected();
+     void reject();
      void testRunRequested();
      
      void append(const QString &line);
