@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ipfAdvancedDialog.h,v 1.1 2004/05/10 01:59:40 vkurland Exp $
+  $Id: ipfAdvancedDialog.h,v 1.2 2004/10/18 05:06:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -49,6 +49,8 @@ protected slots:
     virtual void accept();
     virtual void reject();
 
+    virtual void editProlog();
+    virtual void editEpilog();
 
 };
 
