@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: iptAdvancedDialog.h,v 1.4 2004/10/18 05:06:55 vkurland Exp $
+  $Id: iptAdvancedDialog.h,v 1.4.4.1 2005/09/04 20:49:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,6 +28,7 @@
 #define __IPTADVANCEDDIALOG_H_
 
 #include "iptadvanceddialog_q.h"
+#include "DialogData.h"
 
 namespace libfwbuilder {
     class FWObject;
@@ -38,6 +39,7 @@ class iptAdvancedDialog : public iptAdvancedDialog_q
     Q_OBJECT
 
     libfwbuilder::FWObject *obj;
+    DialogData data;
 
  public:
     iptAdvancedDialog(QWidget *parent,libfwbuilder::FWObject *o);
