@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: listOfLibraries.cpp,v 1.10 2004/08/23 02:34:50 vkurland Exp $
+  $Id: listOfLibraries.cpp,v 1.17 2005/03/21 06:36:47 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -25,10 +25,13 @@
 
 #include "config.h"
 #include "global.h"
+#include "utils.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/FWObjectDatabase.h"
+#include "fwbuilder/XMLTools.h"
 
+#include "FWWindow.h"
 #include "FWBSettings.h"
 #include "upgradePredicate.h"
 #include "listOfLibraries.h"
@@ -38,6 +41,7 @@
 #include <qfile.h>
 
 #include <iostream>
+#include <algorithm>
 
 using namespace std;
 using namespace libfwbuilder;
@@ -50,8 +54,8 @@ listOfLibraries::listOfLibraries()
  * that we usually find but do not load by default (e.g. templates)
  */
 
-    add(sysfname,true);
-    add(tempfname);
+    add(sysfname.c_str(),true);
+    add(tempfname.c_str());
 
 #ifdef _WIN32
     QString dir = appRootDir + "/lib";
@@ -95,7 +99,7 @@ void listOfLibraries::save()
         QString lp;
 
         lp = QString("Libraries/lib%1_path").arg(n);
-        st->setStr( lp , i->path.c_str() );
+        st->setStr( lp , i->path );
 
         lp = QString("Libraries/lib%1_load").arg(n);
         st->setBool( lp , i->load );
@@ -103,104 +107,115 @@ void listOfLibraries::save()
 }
 
 class findPathPredicate {
-    string p;
+    QString p;
     public:
-    findPathPredicate(const string &_p):p(_p){}
+    findPathPredicate(const QString &_p):p(_p){}
     bool operator()(const libData &ld) { return ld.path==p; }
 };
 
 class findNamePredicate {
-    string p;
+    QString p;
     public:
-    findNamePredicate(const string &_p):p(_p){}
+    findNamePredicate(const QString &_p):p(_p){}
     bool operator()(const libData &ld) { return ld.name==p; }
 };
 
 class findIdPredicate {
-    string p;
+    QString p;
     public:
-    findIdPredicate(const string &_p):p(_p){}
+    findIdPredicate(const QString &_p):p(_p){}
     bool operator()(const libData &ld) { return ld.id==p; }
 };
 
-list<libData>::iterator listOfLibraries::add(const string &path, bool load)
+
+
+class MessageBoxNeverUpgradePredicate: public libfwbuilder::XMLTools::UpgradePredicate
 {
-    string name;
-    string id;
+ public:
+    MessageBoxNeverUpgradePredicate() {}
+    
+    virtual bool operator()(const std::string &msg) const 
+    { 
+        QMessageBox::information( NULL , "Firewall Builder", 
+                                  QObject::tr(
+"The library file you are trying to open\n\
+has been saved in an older version of\n\
+Firewall Builder and needs to be upgraded.\n\
+To upgarde it, just load it in the Firewall\n\
+Builder GUI and save back to file again."
+                                  ),
+                                  QObject::tr("&Continue"),
+                                  QString::null,
+                                  QString::null,
+                                  0, 1 );
+        return false;
+    }
+};
 
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//        if (i->path == path) return i;
+list<libData>::iterator listOfLibraries::add(const QString &path, bool load)
+{
+    QString name;
+    QString id;
 
-    list<libData>::iterator it;
-    if ( (it=std::find_if(begin(),end(),findPathPredicate(path)))!=end())
-        return it;
+    if ( ! QFile::exists(path) ) return end();
 
-    if ( ! QFile::exists(path.c_str()) ) return end();
+    MessageBoxNeverUpgradePredicate dont_upgrade_predicate;
 
-/* need to read library file to get library name and ID should not use
- * FWObjectDatabase::load because the library may be from the old
- * version and may need to be upgraded, but if it is in RCS, then
- * upgrade procedure just creates writable file instead of proper RCS
- * checkout 
- */
-#if 0
-    MessageBoxUpgradePredicate upgrade_predicate;
-
-    FWObjectDatabase *ndb = new FWObjectDatabase();
-    ndb->load( path, &upgrade_predicate,librespath);
-    FWObject *lib=ndb->getFirstByType(Library::TYPENAME);
-    if (lib==NULL)
+    try
     {
-        QMessageBox::critical(
+        FWObjectDatabase *ndb = new FWObjectDatabase();
+        ndb->load(path.ascii(),  &dont_upgrade_predicate,  librespath);
+        list<FWObject*> libs = ndb->getByType(Library::TYPENAME);
+        for (list<FWObject*>::iterator i=libs.begin(); i!=libs.end(); i++)
+        {
+            name = (*i)->getName().c_str();
+            id   = (*i)->getId().c_str();
+            if ((*i)->getId() == STANDARD_LIB) continue;
+            if ((*i)->getId() == DELETED_LIB)  continue;
+            if ((*i)->getId() == TEMPLATE_LIB) continue;
+            break;
+        }
+
+        delete ndb;
+
+    } catch(FWException &ex)
+    {
+        QMessageBox::warning(
             NULL,"Firewall Builder", 
-            QObject::tr("Library file %1 is corrupted.").arg(path.c_str()),
-            "&Continue", QString::null, QString::null,
+            QObject::tr("Error loading file %1:\n%2").
+                 arg(path).arg(ex.toString().c_str()),
+            QObject::tr("&Continue"), QString::null,QString::null,
             0, 1 );
         return end();
     }
-    name=lib->getName();
-    id  =lib->getId();
-#endif
 
-    QFile lf(path.c_str());
-    if ( lf.open( IO_ReadOnly) )
+    if (id.isEmpty()) return end();
+    if (name.isEmpty()) return end();
+
+/*
+ *  mw is NULL at this point if this method is called to preload
+ *  libraries on startup
+ */
+    list<FWObject*> currentLibs;
+    if (mw)  currentLibs= mw->db()->getByType(Library::TYPENAME);
+
+    if ( std::find_if(begin(),end(),findIdPredicate(id))!=end() ||
+         (!currentLibs.empty() &&
+          std::find_if(currentLibs.begin(),currentLibs.end(),
+                       findFWObjectIDPredicate(id.ascii()))!=currentLibs.end() )
+    )
     {
-        QTextStream lfs(&lf);
-        QString     s;
-        while ( !lfs.eof() )
-        {
-            s = lfs.readLine();
-
-            if ( s.find( "<Library ", 0)>=0 )
-            {
-                int i1 = s.find( "name=\"", 0 );
-                if (i1<0) continue;
-                i1 += 6;
-                int i2 = s.find( "\"", i1 );
-                name = s.mid(i1,i2-i1).latin1();
-
-                i1 = s.find( "id=\"", 0 );
-                if (i1<0) continue;
-                i1 += 4;
-                i2 = s.find( "\"", i1 );
-                id = s.mid(i1,i2-i1).latin1();
-
-                break;
-            }
-        }
-        lf.close();
+        QMessageBox::warning(
+            NULL,"Firewall Builder", 
+            QObject::tr("Duplicate library '%1'").arg(QString::fromUtf8(name)),
+            QObject::tr("&Continue"), QString::null,QString::null,
+            0, 1 );
+        return end();
     }
 
     if (id == STANDARD_LIB) return end();
     if (id == DELETED_LIB)  return end();
-
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//    {
-//        if (i->id == id )  return i;
-//    }
-
-    if ( (it=std::find_if(begin(),end(),findIdPredicate(id)))!=end())
-        return it;
+    if (id == TEMPLATE_LIB) return end();
 
     list<libData>::iterator i1=insert(end(),libData( id, name, path, false) );
     i1->load=load;
@@ -208,24 +223,15 @@ list<libData>::iterator listOfLibraries::add(const string &path, bool load)
     return i1;
 }
 
-void listOfLibraries::setLoad(const string &libPath, bool f)
+void listOfLibraries::setLoad(const QString &libPath, bool f)
 {
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//    {
-//        if (i->path == libPath) { i->load=f; break; }
-//    }
     list<libData>::iterator it;
     if ( (it=std::find_if(begin(),end(),findPathPredicate(libPath)))!=end())
         it->load=f;
 }
 
-bool listOfLibraries::getLoad(const string &libPath)
+bool listOfLibraries::getLoad(const QString &libPath)
 {
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//    {
-//        if (i->path == libPath) { return (i->load); }
-//    }
-
     list<libData>::iterator it;
     if ( (it=std::find_if(begin(),end(),findPathPredicate(libPath)))!=end())
         return it->load;
@@ -233,13 +239,8 @@ bool listOfLibraries::getLoad(const string &libPath)
     return false;
 }
 
-bool listOfLibraries::isLoaded(const string &libName)
+bool listOfLibraries::isLoaded(const QString &libName)
 {
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//    {
-//        if (i->name == libName ) { return (i->load); }
-//    }
-
     list<libData>::iterator it;
     if ( (it=std::find_if(begin(),end(),findNamePredicate(libName)))!=end())
         return it->load;
@@ -247,13 +248,8 @@ bool listOfLibraries::isLoaded(const string &libName)
     return false;
 }
 
-bool listOfLibraries::isKnown(const string &id)
+bool listOfLibraries::isKnown(const QString &id)
 {
-//    for (list<libData>::iterator i=begin(); i!=end(); ++i)
-//    {
-//        if (i->id == id ) return true;
-//    }
-
     list<libData>::iterator it;
     if ( (it=std::find_if(begin(),end(),findIdPredicate(id)))!=end())
         return it->load;
