@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp,v 1.84 2005/02/14 07:37:56 vkurland Exp $
+  $Id: main.cpp,v 1.89.4.2 2005/09/04 21:54:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -128,6 +128,100 @@ listOfLibraries  *addOnLibs;
 
 #ifndef _WIN32
 
+#ifndef HAVE_CFMAKERAW
+static inline void cfmakeraw(struct termios *termios_p)
+{
+    termios_p->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
+    termios_p->c_oflag &= ~OPOST;
+    termios_p->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    termios_p->c_cflag &= ~(CSIZE|PARENB);
+    termios_p->c_cflag |= CS8;
+}
+#endif
+
+#ifndef HAVE_FORKPTY
+
+#include <unistd.h> 
+#include <stdlib.h> 
+#include <sys/ioctl.h> 
+#include <sys/stream.h> 
+#include <sys/stropts.h> 
+
+/* fork_pty() remplacement for Solaris. 
+* ignore the last two arguments 
+* for the moment 
+*/ 
+int forkpty (int *amaster, char *name, void *unused1, void *unused2)
+{
+    int master, slave; 
+    char *slave_name; 
+    pid_t pid; 
+
+    master = open("/dev/ptmx", O_RDWR); 
+    if (master < 0) 
+        return -1; 
+
+    if (grantpt (master) < 0) 
+    { 
+        close (master); 
+        return -1; 
+    } 
+
+    if (unlockpt (master) < 0) 
+    { 
+        close (master); 
+        return -1; 
+    } 
+
+    slave_name = ptsname (master); 
+    if (slave_name == NULL) 
+    { 
+        close (master); 
+        return -1; 
+    } 
+
+    slave = open (slave_name, O_RDWR); 
+    if (slave < 0) 
+    { 
+        close (master); 
+        return -1; 
+    } 
+
+    if (ioctl (slave, I_PUSH, "ptem") < 0 
+        || ioctl (slave, I_PUSH, "ldterm") < 0) 
+    { 
+        close (slave); 
+        close (master); 
+        return -1; 
+    } 
+
+    if (amaster) 
+        *amaster = master; 
+
+    if (name) 
+        strcpy (name, slave_name); 
+
+    pid = fork (); 
+    switch (pid) 
+    { 
+    case -1: /* Error */ 
+        return -1; 
+    case 0: /* Child */ 
+        close (master); 
+        dup2 (slave, STDIN_FILENO); 
+        dup2 (slave, STDOUT_FILENO); 
+        dup2 (slave, STDERR_FILENO); 
+        return 0; 
+    default: /* Parent */ 
+        close (slave); 
+        return pid; 
+    } 
+
+    return -1; 
+}
+
+#endif
+
 static struct termios save_termios;
 static int            ttysavefd = -1;
 static enum { RESET, RAW, CBREAK } ttystate = RESET;
@@ -137,21 +231,29 @@ int tty_raw(int fd)
     struct termios buf;
 
     if (tcgetattr(fd, &save_termios) < 0)
-        return -1;
+    {
+        qDebug("Can not switch terminal to raw mode, tcgetattr error '%s'",strerror(errno));
+        exit(1);
+    }
 
     buf = save_termios;
 
-    buf.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
-    buf.c_lflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
-    buf.c_lflag &= ~(CSIZE | PARENB);
-    buf.c_lflag |= CS8;
-    buf.c_lflag &= ~(OPOST);
+    cfmakeraw(&buf);
 
-    buf.c_cc[VMIN] = 1;
-    buf.c_cc[VTIME] = 0;
+// this used to use TCSAFLUSH, but that caused stall which I did not
+// completely understand. Apparently there was some data in the output
+// buffer at the moment when we try to switch tty to raw mode, but I
+// could not figure out where this data comes from and why it could
+// not be written to the tty. Anyway, this caused semi-random stalls
+// in the installer because whenever it called fwbuilder -X, the child
+// process would block in this place and stall installer. I had to
+// switch to TCSANOW to fix.
 
-    if (tcsetattr(fd, TCSAFLUSH, &buf) < 0)
-        return -1;
+    if (tcsetattr(fd, TCSANOW, &buf) < 0)
+    {
+        qDebug("Can not switch terminal to raw mode, tcsetattr error '%s'",strerror(errno));
+        exit(1);
+    }
 
     ttystate = RAW;
     ttysavefd = fd;
@@ -165,12 +267,19 @@ int echo_off(int fd)
 
     struct termios stermios;
     if (tcgetattr(fd, &stermios)<0)
-        return -1;
+    {
+        qDebug("Can not turn terminal echo off, tcgetattr error '%s'",strerror(errno));
+        exit(1);
+    }
+
     stermios.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
     stermios.c_oflag &= ~(ONLCR);
 
     if (tcsetattr(fd, TCSANOW, &stermios)<0)
-        return -1;
+    {
+        qDebug("Can not turn terminal echo off, tcsetattr error '%s'",strerror(errno));
+        exit(1);
+    }
 
     return 0;
 }
@@ -183,11 +292,14 @@ ssize_t writen(int fd,const void *vptr, size_t n)
 
     ptr = (const char*)(vptr);
     nleft = n;
+    if (fwbdebug) qDebug("need to write %d bytes",nleft);
     while (nleft > 0)
     {
-        if ( (nwritten = write(fd,ptr,nleft)) <= 0)
+        if ( (nwritten = write(fd,ptr,nleft )) <= 0)
             return nwritten;
 
+        if (fwbdebug) qDebug("%d bytes written",nwritten);
+
         nleft -= nwritten;
         ptr += nwritten;
     }
@@ -319,11 +431,11 @@ int main( int argc, char ** argv )
         { // child
 
 // turn echo off on stdin
-            if (echo_off(STDIN_FILENO) < 0)
-            {
-                qDebug("Can not turn echo off, tcgetattr or tcsetattr error");
-                exit(1);
-            }
+            echo_off(STDIN_FILENO);
+
+            tty_raw(STDIN_FILENO);
+
+            signal(1,SIG_IGN);
 
             execvp(arg[0],(char* const*)arg);
 
@@ -332,10 +444,7 @@ int main( int argc, char ** argv )
             exit(1);
         }
 
-        FILE  *ssh=fdopen(mfd,"w+");
-        setvbuf(ssh,NULL,_IONBF,0);
-
-        tty_raw(STDIN_FILENO);
+        tty_raw(mfd);
 
         fd_set  rfds;
         struct timeval tv;
@@ -383,38 +492,8 @@ int main( int argc, char ** argv )
 // eof on stdin
                         if (fwbdebug) qDebug("EOF on stdin");
                         endOfStream = true;
-//                        if (fwbdebug) qDebug("Flushing ssh");
-//                        int r=fflush(ssh);
-//                        if (r) qDebug("fflush error: %s",strerror(errno));
                     } else
                     {
-// TODO: do not forget to remove this debugging section
-                        if (fwbdebug)
-                        {
-                            int firstLF=-1;
-                            int lastLF=-1;
-                            int i=0;
-                            for (char *c=ibuf; i<n; i++,c++)
-                            {
-                                if (*c=='\n')
-                                {
-                                    lastLF=i;
-                                    if (firstLF==-1) firstLF=i;
-                                }
-                            }
-                            char *firstLine = strndup(ibuf,firstLF+1);
-                            firstLine[firstLF]='\0';
-
-                            int nn = n-lastLF-1;                           
-                            char *lastLine = "";
-                            if (nn!=0)
-                            {
-                                lastLine = strndup(ibuf+lastLF+1,nn);
-                                lastLine[nn]='\0';
-                            }
-                            qDebug("First line: %d: 0-%d  '%s'",firstLF+1,firstLF,firstLine);
-                            qDebug("Last line:  %d: %d-%d '%s'",nn,lastLF,n-1,lastLine);
-                        }
                         int r=writen(mfd,ibuf,n);
                         if (fwbdebug) qDebug("Wrote %d bytes to mfd",r);
                     }
@@ -538,25 +617,33 @@ int main( int argc, char ** argv )
 
         if (fwbdebug) qDebug("loading translation for the current locale ...");
 
-        QTranslator qt1(0);
-        qt1.load( QString( "qt_" ) + QTextCodec::locale(), QTTRANSLATIONSDIR );
-        app->installTranslator( &qt1 );
-
-        QTranslator qt2(0);
-        qt2.load( QString( "qt_" ) + QTextCodec::locale(),  localepath.c_str() );
-        app->installTranslator( &qt2 );
+/* this is an ugly hack to work around broken qt_es.qm translation
+ * file shipped with QT 3.3 for Fedora-C4 and possibly other
+ * distros. Only Spanish translation seems to be affected.  Trolltech
+ * support req. N80793 (although I never got any suggestions for the
+ * workaround, nor did they clearly admit the problem despite my
+ * sending a simple example program to them to illustrate the
+ * bug). They did not seem to care since Spanish translation file that
+ * support guy had on his computer was fine, but he admitted it was
+ * different from the one shipped with QT 3.3.4 for Fedora-C4 and for
+ * Mac OS X. All I got from them was a promise to look into packaging
+ * process.
+ */
+        if (QString(QTextCodec::locale()).find("es")!=0)
+        {
+            QTranslator qt1(0);
+            qt1.load( QString( "qt_" ) + QTextCodec::locale(), QTTRANSLATIONSDIR );
+            app->installTranslator( &qt1 );
+        
+            QTranslator qt2(0);
+            qt2.load( QString( "qt_" ) + QTextCodec::locale(),  localepath.c_str() );
+            app->installTranslator( &qt2 );
+        }
 
         QTranslator translator(0);
         translator.load(QString("fwbuilder_")+QTextCodec::locale(),localepath.c_str());
         app->installTranslator (&translator);
 
-#if 0
-        st = new FWBSettings();
-
-/* initialize preferences */
-        st->init();
-#endif
-
 /* must build list of available libraries _after_ creation of
  * FWObjectDatabase and settings */
 
