@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pfAdvancedDialog.cpp,v 1.16 2005/04/10 07:16:39 vkurland Exp $
+  $Id: pfAdvancedDialog.cpp,v 1.16.4.1 2005/09/04 20:49:14 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -52,6 +52,7 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     : pfAdvancedDialog_q(parent)
 {
     obj=o;
+    QStringList slm;
 
     FWOptions *fwopt=(Firewall::cast(obj))->getOptionsObject();
     assert(fwopt!=NULL);
@@ -80,7 +81,22 @@ pfAdvancedDialog::pfAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( pf_limit_frags         ,fwopt, "pf_limit_frags"      );
     data.registerOption( pf_do_limit_states     ,fwopt, "pf_do_limit_states"     );
     data.registerOption( pf_limit_states        ,fwopt, "pf_limit_states"     );
-    data.registerOption( pf_optimization        ,fwopt, "pf_optimization"     );
+// Prepare mapping for pf_optimization:
+    slm.clear();
+    slm.push_back("");
+    slm.push_back("");
+    slm.push_back(QObject::tr("Aggressive"));
+    slm.push_back("Aggressive");
+    slm.push_back(QObject::tr("Conservative"));
+    slm.push_back("Conservative");
+    slm.push_back(QObject::tr("For high latency"));
+    slm.push_back("For high latency");
+    slm.push_back(QObject::tr("Normal"));
+    slm.push_back("Normal");
+    pf_optimization->clear();
+    pf_optimization->insertStringList(getScreenNames(slm));
+    data.registerOption( pf_optimization,   fwopt,      "pf_optimization", slm); 
+    
     data.registerOption( pf_check_shadowing     ,fwopt, "check_shading"       );
     data.registerOption( pf_pass_all_out        ,fwopt, "pass_all_out"        );
     data.registerOption( pf_in_out_code         ,fwopt, "in_out_code"         );
