@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pfAdvancedDialog.h,v 1.4 2004/06/05 17:01:02 vkurland Exp $
+  $Id: pfAdvancedDialog.h,v 1.5 2004/10/18 05:06:55 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,8 +48,11 @@ protected slots:
 
     virtual void accept();
     virtual void reject();
+
     virtual void doScrubToggled();
     virtual void ltToggled();
+    virtual void editProlog();
+    virtual void editEpilog();
 
 };
 
