@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pixAdvancedDialog.cpp,v 1.18 2005/04/10 07:16:39 vkurland Exp $
+  $Id: pixAdvancedDialog.cpp,v 1.18.4.1 2005/09/13 04:55:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -62,6 +62,7 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     obj=o;
 
     string vers="version_"+obj->getStr("version");
+    string platform = obj->getStr("platform");   // could be 'pix' or 'fwsm'
 
     QString      s;
     QStringList  logLevels;
@@ -220,6 +221,10 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     data.registerOption( pix_regroup_commands , fwoptions,  
                          "pix_regroup_commands" );
 
+    data.registerOption( pix_use_manual_commit , fwoptions,  
+                         "pix_use_manual_commit" );
+
+    pix_use_manual_commit->setEnabled(platform=="fwsm");
 /*
     data.registerOption( pix_add_clear_statements , fwoptions,  
                          "pix_add_clear_statements" );
@@ -333,7 +338,7 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
     allFixups.push_back(fixupControl( pix_sqlnet_switch,   pix_sqlnet_port1,      pix_sqlnet_port2,   NULL,             "sqlnet_fixup"    , "sqlnet"    , 17));
     allFixups.push_back(fixupControl( pix_tftp_switch,     pix_tftp_port,         NULL,               NULL,             "tftp_fixup"      , "tftp"      , 18));
 
-    string lst=Resources::platform_res["pix"]->getResourceStr(
+    string lst=Resources::platform_res[platform]->getResourceStr(
             "/FWBuilderResources/Target/options/"+vers+"/fixups/list");
 
     for (list<fixupControl>::iterator fi=allFixups.begin(); fi!=allFixups.end(); fi++)
@@ -370,11 +375,11 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
 /* page Logging */
 
     emblem_log_format->setEnabled(
-        Resources::platform_res["pix"]->getResourceBool(
+        Resources::platform_res[platform]->getResourceBool(
             "/FWBuilderResources/Target/options/"+vers+"/pix_emblem_log_format") );
 
 
-    syslogDeviceIdSupported= (Resources::platform_res["pix"]->getResourceBool(
+    syslogDeviceIdSupported= (Resources::platform_res[platform]->getResourceBool(
                   "/FWBuilderResources/Target/options/"+vers+"/pix_syslog_device_id_supported") );
 
     syslog_device_id_hostname->setEnabled(syslogDeviceIdSupported);
@@ -449,11 +454,11 @@ pixAdvancedDialog::pixAdvancedDialog(QWidget *parent,FWObject *o)
 /* page Options */
 
     fragguard->setEnabled(
-        Resources::platform_res["pix"]->getResourceBool(
+        Resources::platform_res[platform]->getResourceBool(
             "/FWBuilderResources/Target/options/"+vers+"/pix_security_fragguard_supported") );
 
     route_dnat->setEnabled(
-        Resources::platform_res["pix"]->getResourceBool(
+        Resources::platform_res[platform]->getResourceBool(
             "/FWBuilderResources/Target/options/"+vers+"/pix_route_dnat_supported") );
 
     data.registerOption( fragguard,           fwoptions,  "pix_fragguard"           );
@@ -752,6 +757,7 @@ void pixAdvancedDialog::editEpilog()
 
 void pixAdvancedDialog::setDefaultTimeoutValue(const QString &option)
 {
+    string platform = obj->getStr("platform");   // could be 'pix' or 'fwsm'
     FWOptions *fwoptions=(Firewall::cast(obj))->getOptionsObject();
     assert(fwoptions!=NULL);
     
@@ -759,11 +765,11 @@ void pixAdvancedDialog::setDefaultTimeoutValue(const QString &option)
 
     if (option=="uauth_abs" || option=="uauth_inact")
         data.setWidgetValue(option,
-                        Resources::platform_res["pix"]->getResourceBool(
+                        Resources::platform_res[platform]->getResourceBool(
      "/FWBuilderResources/Target/options/"+vers+"/timeouts/"+option.latin1()));
     else
         data.setWidgetValue(option,
-                        Resources::platform_res["pix"]->getResourceInt(
+                        Resources::platform_res[platform]->getResourceInt(
      "/FWBuilderResources/Target/options/"+vers+"/timeouts/"+option.latin1()));
 }
 
