@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.cpp,v 1.12 2005/01/31 06:10:07 vkurland Exp $
+  $Id: platforms.cpp,v 1.12.4.2 2005/09/13 04:55:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -43,7 +43,7 @@ using namespace libfwbuilder;
 bool isUsingNetZone(Firewall *fw)
 {
     string platform=fw->getStr("platform");
-    return (platform=="pix");
+    return (platform=="pix" || platform=="fwsm");
 }
 
 bool isDefaultOptions(FWOptions *opt)
@@ -82,7 +82,7 @@ bool isDefaultOptions(FWOptions *opt)
                    ! opt->getBool("firewall_is_part_of_any_and_networks"));
 	}
 
-	if (platform=="pix") 
+	if (platform=="pix" || platform=="fwsm") 
         {
             res= ( opt->getStr("log_level").empty()        && 
                    opt->getInt("log_interval")<=0          &&
@@ -130,9 +130,9 @@ QMap<QString,QString> getVersionsForPlatform(const QString &platform)
         res["1.2.9"]    = QObject::tr("1.2.9 or later");
     } else
     {
-        if (platform=="pix") 
+        if (platform=="pix" || platform=="fwsm") 
         {
-            QString lst=Resources::platform_res["pix"]->getResourceStr(
+            QString lst=Resources::platform_res[platform]->getResourceStr(
                 "/FWBuilderResources/Target/versions").c_str();
 
             QStringList ll=QStringList::split(',', lst);
@@ -173,13 +173,21 @@ QStringList getLogLevels(const QString &platform)
     QStringList logLevels;
 
     logLevels.push_back("");
-    logLevels.push_back("alert");
-    logLevels.push_back("crit");
-    logLevels.push_back("error");
-    logLevels.push_back("warning");
-    logLevels.push_back("notice");
-    logLevels.push_back("info");
-    logLevels.push_back("debug");
+    logLevels.push_back("");
+    logLevels.push_back(QObject::tr( "alert"  ));
+    logLevels.push_back(             "alert"  );
+    logLevels.push_back(QObject::tr( "crit"   ));
+    logLevels.push_back(             "crit"   );
+    logLevels.push_back(QObject::tr( "error"  ));
+    logLevels.push_back(             "error"  );
+    logLevels.push_back(QObject::tr( "warning"));
+    logLevels.push_back(             "warning");
+    logLevels.push_back(QObject::tr( "notice" ));
+    logLevels.push_back(             "notice" );
+    logLevels.push_back(QObject::tr( "info"   ));
+    logLevels.push_back(             "info"   );
+    logLevels.push_back(QObject::tr( "debug"  ));
+    logLevels.push_back(             "debug"  );
 
     return logLevels;
 }
@@ -189,29 +197,118 @@ QStringList getLogFacilities(const QString &platform)
     QStringList logFacilities;
 
     logFacilities.push_back("");
-    logFacilities.push_back("kern");
-    logFacilities.push_back("user");
-    logFacilities.push_back("mail");
-    logFacilities.push_back("daemon");
-    logFacilities.push_back("auth");
-    logFacilities.push_back("syslog");
-    logFacilities.push_back("lpr");
-    logFacilities.push_back("news");
-    logFacilities.push_back("uucp");
-    logFacilities.push_back("cron");
-    logFacilities.push_back("authpriv");
-    logFacilities.push_back("ftp");
-    logFacilities.push_back("local0");
-    logFacilities.push_back("local1");
-    logFacilities.push_back("local2");
-    logFacilities.push_back("local3");
-    logFacilities.push_back("local4");
-    logFacilities.push_back("local5");
-    logFacilities.push_back("local6");
-    logFacilities.push_back("local7");
-
+    logFacilities.push_back("");
+    logFacilities.push_back(QObject::tr( "kern"   ));
+    logFacilities.push_back(             "kern"   );
+    logFacilities.push_back(QObject::tr( "user"   ));
+    logFacilities.push_back(             "user"   );
+    logFacilities.push_back(QObject::tr( "mail"   ));
+    logFacilities.push_back(             "mail"   );
+    logFacilities.push_back(QObject::tr( "daemon" ));
+    logFacilities.push_back(             "daemon" );
+    logFacilities.push_back(QObject::tr( "auth"   ));
+    logFacilities.push_back(             "auth"   );
+    logFacilities.push_back(QObject::tr( "syslog" ));
+    logFacilities.push_back(             "syslog" );
+    logFacilities.push_back(QObject::tr( "lpr"    ));
+    logFacilities.push_back(             "lpr"    );
+    logFacilities.push_back(QObject::tr( "news"   ));
+    logFacilities.push_back(             "news"   );
+    logFacilities.push_back(QObject::tr( "uucp"   ));
+    logFacilities.push_back(             "uucp"   );
+    logFacilities.push_back(QObject::tr( "cron"   ));
+    logFacilities.push_back(             "cron"   );
+    logFacilities.push_back(QObject::tr( "authpriv" ));
+    logFacilities.push_back(             "authpriv");
+    logFacilities.push_back(QObject::tr( "ftp"    ));
+    logFacilities.push_back(             "ftp"    );
+    logFacilities.push_back(QObject::tr( "local0" ));
+    logFacilities.push_back(             "local0" );
+    logFacilities.push_back(QObject::tr( "local1" ));
+    logFacilities.push_back(             "local1" );
+    logFacilities.push_back(QObject::tr( "local2" ));
+    logFacilities.push_back(             "local2" );
+    logFacilities.push_back(QObject::tr( "local3" ));
+    logFacilities.push_back(             "local3" );
+    logFacilities.push_back(QObject::tr( "local4" ));
+    logFacilities.push_back(             "local4" );
+    logFacilities.push_back(QObject::tr( "local5" ));
+    logFacilities.push_back(             "local5" );
+    logFacilities.push_back(QObject::tr( "local6" ));
+    logFacilities.push_back(             "local6" );
+    logFacilities.push_back(QObject::tr( "local7" ));
+    logFacilities.push_back(             "local7" );
+    
     return logFacilities;
 }
+QStringList getActionsOnReject(const QString &platform)
+{
+    QStringList slm;
+    
+    slm.push_back("");
+    slm.push_back("");
+    slm.push_back(QObject::tr("ICMP admin prohibited"));
+    slm.push_back("ICMP admin prohibited");
+    slm.push_back(QObject::tr("ICMP host prohibited"));
+    slm.push_back("ICMP host prohibited");
+    slm.push_back(QObject::tr("ICMP host unreachable"));
+    slm.push_back("ICMP host unreachable");
+    slm.push_back(QObject::tr("ICMP net prohibited"));
+    slm.push_back("ICMP net prohibited");
+    slm.push_back(QObject::tr("ICMP net unreachable"));
+    slm.push_back("ICMP net unreachable");
+    slm.push_back(QObject::tr("ICMP port unreachable"));
+    slm.push_back("ICMP port unreachable");
+    slm.push_back(QObject::tr("ICMP protocol unreachable"));
+    slm.push_back("ICMP protocol unreachable");
+    slm.push_back(QObject::tr("TCP RST"));
+    slm.push_back("TCP RST");
+
+    return slm;
+}
+QStringList getPrologPlaces(const QString &platform)
+{
+    QStringList slm;
+    
+    slm.push_back(QObject::tr("on top of the script"));
+    slm.push_back("top");
+    slm.push_back(QObject::tr("after interface configuration"));
+    slm.push_back("after_interfaces");
+    slm.push_back(QObject::tr("after policy reset"));
+    slm.push_back("after_flush");
+
+    return slm;
+}
+
+QStringList getLimitSuffixes(const QString &platform)
+{
+    QStringList slm;
+    
+    slm.push_back("");
+    slm.push_back("");
+    slm.push_back(QObject::tr("/day"));
+    slm.push_back("/day");
+    slm.push_back(QObject::tr("/hour"));
+    slm.push_back("/hour");
+    slm.push_back(QObject::tr("/minute"));
+    slm.push_back("/minute");
+    slm.push_back(QObject::tr("/second"));
+    slm.push_back("/second");
+
+    return slm;
+}
 
 
+QStringList getScreenNames(QStringList sl)
+{
+    QStringList res;
+    
+    for(    QStringList::iterator it = sl.begin(); 
+            it!=sl.end();
+            ++it,++it)
+    {
+        res.push_back(*it);
+    }
+    return res;
+}
 
