@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.cpp,v 1.38 2004/08/02 01:56:21 vkurland Exp $
+  $Id: utils.cpp,v 1.43 2005/03/21 06:36:47 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -50,10 +50,12 @@
 #  include <sys/stat.h>
 #  include <fcntl.h>
 #  include <time.h>
+#  include <windows.h>
 #else
 #  include <unistd.h>
 #  include <string.h>
 #  include <errno.h>
+#  include <pwd.h>
 #endif
 
 #include <iostream>
@@ -75,7 +77,7 @@ class mapValEQPredicate {
 void addPopupMenuItem(QObject *res,
                       QPopupMenu* menu,
                       const QString &resourceIconPath,
-                      const char* itemName,
+                      const QString itemName,
                       const char* member,
                       const QKeySequence &accel)
 {
@@ -93,8 +95,21 @@ void addPopupMenuItem(QObject *res,
 
 void fillLibraries(QComboBox *libs, libfwbuilder::FWObject *obj, bool rw)
 {
+    bool standardObj = false;
+    bool templateObj = false;
+    bool deletedObj = false;
     QString lib="";
-    if ( ! FWObjectDatabase::isA(obj)) lib=obj->getLibraryName().c_str();
+
+    if ( ! FWObjectDatabase::isA(obj))
+    {
+        FWObject *libobj = obj->getLibrary();
+        assert(libobj!=NULL);
+        lib = libobj->getName().c_str();
+        standardObj = (libobj->getId()==STANDARD_LIB);
+        templateObj = (libobj->getId()==TEMPLATE_LIB);
+        deletedObj = (libobj->getId()==DELETED_LIB);
+    }
+
     libs->clear();
     list<FWObject*> ll = obj->getRoot()->getByType( Library::TYPENAME );
     ll.sort(FWObjectNameCmpPredicate());
@@ -106,14 +121,14 @@ void fillLibraries(QComboBox *libs, libfwbuilder::FWObject *obj, bool rw)
         if (libicn.empty())
             libicn=Resources::global_res->getObjResourceStr(*i,"icon-tree").c_str();
 
-        if ( (*i)->getId()==STANDARD_LIB ||
-             (*i)->getId()==TEMPLATE_LIB ||
-             (*i)->getId()==DELETED_LIB )    continue;
+        if ( (*i)->getId()==STANDARD_LIB && !standardObj) continue;
+        if ( (*i)->getId()==TEMPLATE_LIB && !templateObj) continue;
+        if ( (*i)->getId()==DELETED_LIB  && !deletedObj ) continue;
 
         if (rw && (*i)->isReadOnly()) continue;
         if (lib==(*i)->getName().c_str()) cn=n;
         libs->insertItem(QPixmap::fromMimeSource( libicn.c_str() ),
-                         (*i)->getName().c_str() );
+                         QString::fromUtf8((*i)->getName().c_str()) );
         n++;
     }
     libs->setCurrentItem(cn);
@@ -122,8 +137,21 @@ void fillLibraries(QComboBox *libs, libfwbuilder::FWObject *obj, bool rw)
 
 void fillLibraries(QListBox *libs, libfwbuilder::FWObject *obj, bool rw)
 {
+    bool standardObj = false;
+    bool templateObj = false;
+    bool deletedObj = false;
     QString lib="";
-    if ( ! FWObjectDatabase::isA(obj)) lib=obj->getLibraryName().c_str();
+
+    if ( ! FWObjectDatabase::isA(obj))
+    {
+        FWObject *libobj = obj->getLibrary();
+        assert(libobj!=NULL);
+        lib = libobj->getName().c_str();
+        standardObj = (libobj->getId()==STANDARD_LIB);
+        templateObj = (libobj->getId()==TEMPLATE_LIB);
+        deletedObj = (libobj->getId()==DELETED_LIB);
+    }
+
     libs->clear();
     list<FWObject*> ll = obj->getRoot()->getByType( Library::TYPENAME );
     ll.sort(FWObjectNameCmpPredicate());
@@ -135,9 +163,9 @@ void fillLibraries(QListBox *libs, libfwbuilder::FWObject *obj, bool rw)
         if (libicn.empty())
             libicn=Resources::global_res->getObjResourceStr(*i,"icon-tree").c_str();
 
-        if ( (*i)->getId()==STANDARD_LIB ||
-             (*i)->getId()==TEMPLATE_LIB ||
-             (*i)->getId()==DELETED_LIB )    continue;
+        if ( (*i)->getId()==STANDARD_LIB && !standardObj) continue;
+        if ( (*i)->getId()==TEMPLATE_LIB && !templateObj) continue;
+        if ( (*i)->getId()==DELETED_LIB  && !deletedObj ) continue;
 
         if (rw && (*i)->isReadOnly()) continue;
         if (lib==(*i)->getName().c_str()) cn=n;
@@ -198,16 +226,22 @@ QString quoteString(const QString &str)
 
 QString getUserName()
 {
+    QString uname;
+
 #ifdef _WIN32
-/* also need to set env variable USER for rcs tools, but if the user name
- * contains spaces, replace them with underscores (like "John Smith")
- */
-    QString uname=getenv("USERNAME");
+
+#define INFO_BUFFER_SIZE 32767
+TCHAR  infoBuf[INFO_BUFFER_SIZE];
+DWORD  bufCharCount = INFO_BUFFER_SIZE;
+
+  bufCharCount = INFO_BUFFER_SIZE;
+  if( GetUserName( infoBuf, &bufCharCount ) )
+    uname=qt_winQString( infoBuf );
     uname=uname.replace(' ','_');
 #else
-    QString uname=getenv("USER");
-    if (uname.isEmpty()) uname=getenv("LOGNAME");
-    if (uname.isEmpty()) uname=getlogin();
+    struct passwd *pwd=getpwuid(getuid());
+    assert(pwd);
+    uname=QString(pwd->pw_name);
 #endif
     return uname;
 }
@@ -218,7 +252,10 @@ QString getFileDir(const QString &file)
     int sn1  = file.findRev('/',-1);
     int sn2  = file.findRev('\\',-1);
     int sn   = (sn1>=0)?sn1:sn2;
-    QString dir = (sn<0) ? "./" : file.left( sn );
+    QString dir;
+
+    if(sn<0) dir = "./";
+    else     dir = file.left( sn );
 
 #ifdef _WIN32
 /* on windows, if directory is in the root of the drive (like "c:"),
