@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.h,v 1.22 2004/06/13 07:45:51 vkurland Exp $
+  $Id: utils.h,v 1.24 2004/12/11 07:54:30 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,11 +53,19 @@ struct FWObjectNameCmpPredicate :
     }
 };
 
+class findFWObjectIDPredicate: public std::unary_function<libfwbuilder::FWObject*, bool>
+{
+    std::string _id;
+    public:
+    findFWObjectIDPredicate(const std::string &id):_id(id) {}
+    bool operator()(const libfwbuilder::FWObject *o) const {return o->getId()==_id;}
+};
+
 
 extern void addPopupMenuItem(QObject *res,
                              QPopupMenu* menu,
                              const QString &resourceIconPath,
-                             const char* itemName,
+                             const QString itemName,
                              const char* member,
                              const QKeySequence &accel = 0);
 
