@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipf.cpp,v 1.18 2005/04/14 06:30:37 vkurland Exp $
+  $Id: ipf.cpp,v 1.19 2005/05/29 03:13:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -462,7 +462,7 @@ int main(int argc, char * const *argv)
 
         fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
-               << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
+               << timestr << " " << _(" by ")
                << user_name;
         fw_file << "'" << endl;
 
