@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipfw.cpp,v 1.15 2005/04/14 06:30:37 vkurland Exp $
+  $Id: ipfw.cpp,v 1.18 2005/05/30 22:09:19 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -417,7 +417,7 @@ int main(int argc, char * const *argv)
 
         fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
-               << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
+               << timestr << " " << _(" by ")
                << user_name;
         fw_file << "'" << endl;
 
@@ -425,8 +425,9 @@ int main(int argc, char * const *argv)
 
 
 
-
-        fw_file << "$IPFW -f flush" << endl;
+/* commented out since we now use sets
+        fw_file << "\"$IPFW\" -f -q flush" << endl;
+*/
         fw_file << endl;
 
 	if (have_ipfw)
@@ -446,6 +447,9 @@ int main(int argc, char * const *argv)
         fw_file << "# End of epilog script" << endl;
         fw_file << "#" << endl;
 
+        fw_file << "\"$IPFW\" set swap 0 1 || exit 1" << endl;
+        fw_file << "\"$IPFW\" delete set 1" << endl;
+
 	fw_file << endl;
 	fw_file.close();
 
