@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.cpp,v 1.7 2004/09/09 05:36:56 vkurland Exp $
+  $Id: NATCompiler_ipt.cpp,v 1.11 2004/12/29 07:56:23 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -73,6 +73,17 @@ static std::map<std::string,int> tmp_chain_no;
 
 string NATCompiler_ipt::myPlatformName() { return "iptables"; }
 
+string NATCompiler_ipt::getInterfaceVarName(FWObject *iface)
+{
+    ostringstream  ostr;
+    string iname=iface->getName();
+    string::size_type p1;
+    while ( (p1=iname.find("."))!=string::npos)
+        iname=iname.replace(p1,1,"_");
+    ostr << "i_" << iname;
+    return ostr.str();
+}
+
 string NATCompiler_ipt::getNewTmpChainName(NATRule *rule)
 {
     std::ostringstream str;
@@ -255,7 +266,7 @@ bool NATCompiler_ipt::splitSDNATRule::processNext()
 
     if ( rule->getRuleType()==NATRule::SDNAT)
     {
-        RuleElementOSrc *osrc;
+//        RuleElementOSrc *osrc;
         RuleElementODst *odst;
         RuleElementOSrv *osrv;
         RuleElementTSrc *tsrc;
@@ -977,7 +988,7 @@ bool NATCompiler_ipt::dynamicInterfaceInTSrc::processNext()
 
     if (rule->getRuleType()==NATRule::SNAT && 
         Interface::cast(tsrc)!=NULL &&
-	Interface::cast(tsrc)->isDyn()) 
+        (Interface::cast(tsrc)->isDyn() || Interface::cast(tsrc)->isUnnumbered()))
     {
 
             rule->setRuleType(NATRule::Masq);
@@ -1389,6 +1400,7 @@ bool NATCompiler_ipt::doOSrcNegation::processNext()
 //	ntdst=r->getTDst();  ntdst->clearChildren();  ntdst->setAnyElement();
 //	r->setRuleType(NATRule::Continue);
 	r->setStr("ipt_target",new_chain);
+//	r->setBool("rule_added_for_osrc_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   A  any  any  RETURN  */
@@ -1406,6 +1418,7 @@ bool NATCompiler_ipt::doOSrcNegation::processNext()
 	r->setStr("ipt_target","RETURN");
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+	r->setBool("rule_added_for_osrc_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   any any C  ACTION  */
@@ -1419,6 +1432,7 @@ bool NATCompiler_ipt::doOSrcNegation::processNext()
 	nsrv->setNeg(false);
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+	r->setBool("rule_added_for_osrc_neg",true);
 	tmp_queue.push_back(r);
 
     } else
@@ -1464,6 +1478,7 @@ bool NATCompiler_ipt::doODstNegation::processNext()
 //	ntdst=r->getTDst();  ntdst->clearChildren();  ntdst->setAnyElement();
 //	r->setRuleType(NATRule::Continue);
 	r->setStr("ipt_target",new_chain);
+	r->setBool("rule_added_for_odst_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   any  B  any  RETURN  */
@@ -1481,6 +1496,7 @@ bool NATCompiler_ipt::doODstNegation::processNext()
 	r->setStr("ipt_target","RETURN");
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+//	r->setBool("rule_added_for_odst_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   any  any  C  ACTION  */
@@ -1494,6 +1510,7 @@ bool NATCompiler_ipt::doODstNegation::processNext()
 	nsrv->setNeg(false);
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+	r->setBool("rule_added_for_odst_neg",true);
 	tmp_queue.push_back(r);
 
     } else
@@ -1538,6 +1555,7 @@ bool NATCompiler_ipt::doOSrvNegation::processNext()
 //	ntdst=r->getTDst();  ntdst->clearChildren();  ntdst->setAnyElement();
 //	r->setRuleType(NATRule::Continue);
 	r->setStr("ipt_target",new_chain);
+	r->setBool("rule_added_for_osrv_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   any  any  C  RETURN  */
@@ -1554,6 +1572,7 @@ bool NATCompiler_ipt::doOSrvNegation::processNext()
 	r->setStr("ipt_target","RETURN");
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+	r->setBool("rule_added_for_osrv_neg",true);
 	tmp_queue.push_back(r);
 
 /* TMP_CHAIN   any  any  any  ACTION  */
@@ -1567,6 +1586,7 @@ bool NATCompiler_ipt::doOSrvNegation::processNext()
 	ndst->setNeg(false);
 	r->setStr("ipt_chain",new_chain);
 	r->setInterfaceStr("nil");
+//	r->setBool("rule_added_for_osrv_neg",true);
 	tmp_queue.push_back(r);
 
     } else
@@ -1582,35 +1602,94 @@ bool NATCompiler_ipt::splitNONATRule::processNext()
     if ( rule->getStr("ipt_chain").empty() && rule->getRuleType()==NATRule::NONAT) {
 
         Address *osrc=compiler->getFirstOSrc(rule);
-        if ( compiler->complexMatch(osrc,compiler->fw) ) 
+        bool osrcfw= compiler->complexMatch(osrc,compiler->fw);
+/*
+ * NONAT is special if OSrc matches firewall. It is not sufficient to
+ * only put this rule in the OUTPUT chain because packets originating
+ * on the firewall actually cross both OUTPUT and POSTROUTING chains
+ * (I tested this). So, we need to make sure we _do not_ translate in
+ * both these chains because there may be other rules in POSTROUTING
+ * chain that may accidentally match the packet and translate it.
+ */
+        NATRule *r= NATRule::cast(compiler->dbcopy->create(NATRule::TYPENAME,true) );
+        compiler->temp_ruleset->add(r);
+        r->duplicate(rule);
+        r->setStr("ipt_chain","POSTROUTING");  
+        tmp_queue.push_back(r);
+
+        if (osrcfw)
         {
-            rule->setStr("ipt_chain","OUTPUT");  
-            RuleElement *rel=rule->getOSrc();
-	    rel->clearChildren();
-	    rel->setAnyElement();
-            tmp_queue.push_back(rule);
-        } else 
-        {
-            rule->setStr("ipt_chain","PREROUTING");  
-            tmp_queue.push_back(rule);
+            rule->setStr("ipt_chain","OUTPUT");
+            if (osrc->getId()==compiler->fw->getId())
+            {
+                RuleElementOSrc *src;
+                src=rule->getOSrc();
+                src->clearChildren();
+                src->setAnyElement();
+            }
+        } else          rule->setStr("ipt_chain","PREROUTING");  
+
+        tmp_queue.push_back(rule);
 
-            NATRule *r= NATRule::cast(compiler->dbcopy->create(NATRule::TYPENAME,true) );
-            compiler->temp_ruleset->add(r);
-            r->duplicate(rule);
-            r->setStr("ipt_chain","POSTROUTING");  
-            tmp_queue.push_back(r);
-        }
     } else
         tmp_queue.push_back(rule);
     return true;
 }
 
+bool NATCompiler_ipt::localNATRule::processNext()
+{
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+
+//    if ( rule->getStr("ipt_chain").empty())
+//    {
+        Address *osrc=compiler->getFirstOSrc(rule);
+        bool osrcfw= compiler->complexMatch(osrc,compiler->fw);
+
+        switch( rule->getRuleType())
+        {
+        case NATRule::DNAT:
+        case NATRule::DNetnat:
+
+/* it should not be necessary to do anything if rule type is NONAT
+ * since splitNONATRule takes care of NONAT rules
+ *
+ * is there any need to split the rule if it is SNAT or DNAT type ?  I
+ * can't see any reason to do it.
+ *
+ * Can use OUTPUT chain only for DNAT rules and a like
+ */
+            if (osrcfw) rule->setStr("ipt_chain","OUTPUT");
+            if (osrcfw && osrc->getId()==compiler->fw->getId())
+            {
+                RuleElementOSrc *src;
+                src=rule->getOSrc();
+                src->clearChildren();
+                src->setAnyElement();
+            }
+            break;
+        default:
+            break;
+        }
+//    }
+
+    tmp_queue.push_back(rule);
+
+    return true;
+}
+
 bool NATCompiler_ipt::splitIfOSrcAny::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
 
     tmp_queue.push_back(rule);
 
+/* do not split rules added to handle negation, these rules have "any"
+ * in OSrc but get control only after OSrc is tested by another
+ * rule */
+    if (rule->getBool("rule_added_for_osrc_neg")) return true;
+    if (rule->getBool("rule_added_for_odst_neg")) return true;
+    if (rule->getBool("rule_added_for_osrv_neg")) return true;
+
     if (rule->getRuleType()==NATRule::DNAT)
     {
 //        RuleElementOSrc *osrcrel=rule->getOSrc();
@@ -1960,44 +2039,56 @@ void NATCompiler_ipt::compile()
         }
 
         add( new splitNONATRule("process NAT rules that request no translation"));
-        add( new DNATforFW("process DNAT rules for packets originated on the firewall"));
+        add( new localNATRule("process local NAT rules"));
+//        add( new DNATforFW("process DNAT rules for packets originated on the firewall"));
         add( new decideOnChain(    "decide on chain"                    ) );
         add( new decideOnTarget(   "decide on target"                   ) );
 
         add( new splitODstForSNAT( "split rule if objects in ODst belong to different subnets") );
-        add( new ReplaceFirewallObjectsODst(   "replace firewall in ODst" ) );
-	add( new ReplaceFirewallObjectsTSrc(   "replace firewall in TSrc" ) );
+        add( new ReplaceFirewallObjectsODst("replace firewall in ODst" ) );
+	add( new ReplaceFirewallObjectsTSrc("replace firewall in TSrc" ) );
         add( new splitOnDynamicInterfaceInODst("split rule if ODst is dynamic interface" ) );
         add( new splitOnDynamicInterfaceInTSrc("split rule if TSrc is dynamic interface" ) );
 
-        add( new ExpandMultipleAddresses(      "expand multiple addresses") );
+        add( new ExpandMultipleAddresses("expand multiple addresses") );
 
         add( new specialCaseWithUnnumberedInterface("check for a special cases with dynamic and unnumbered interfaces" ) );
         add( new checkForDynamicInterfacesOfOtherObjects( "check for dynamic interfaces of other hosts and firewalls" ) );
 
-        add( new verifyRuleWithMAC(                 "verify rules using MAC address filtering" ) );
-        add( new ExpandAddressRanges(               "expand address ranges"              ) );
+        add( new verifyRuleWithMAC("verify rules using MAC address filtering"));
+        add( new ExpandAddressRanges("expand address ranges") );
 
-        add( new splitMultiSrcAndDst(          "split rules where multiple srcs and dsts are present" ) );
+        add( new splitMultiSrcAndDst("split rules where multiple srcs and dsts are present" ) );
 
-        add( new splitServices(                     "split on services"                  ) );
-        add( new VerifyRules2(                      "check correctness of TSrv"          ) );
-        add( new separatePortRanges(                "separate port ranges"               ) );
-        add( new separateSourcePorts(               "separate objects with src"          ) );
+        add( new splitServices("split on services") );
+        add( new VerifyRules2("check correctness of TSrv") );
+        add( new separatePortRanges("separate port ranges") );
+        add( new separateSourcePorts("separate objects with src") );
         add( new separateSourceAndDestinationPorts( "separate objects with both src and dest ports" ) );
-        add( new prepareForMultiport(               "prepare for multiport"              ) );
-        add( new splitMultipleICMP(                 "split rule with multiple ICMP services" ) );
+        add( new prepareForMultiport("prepare for multiport") );
+        add( new splitMultipleICMP("split rule with multiple ICMP services") );
 
-        add( new ConvertToAtomicForAddresses(       "convert to atomic rules"            ) );
+        add( new ConvertToAtomicForAddresses("convert to atomic rules") );
 
-        add( new addVirtualAddress(                 "add virtual addresses"              ) );
+        add( new addVirtualAddress("add virtual addresses") );
 
-        add( new AssignInterface(                   "assign rules to interfaces"         ) );
-        add( new dynamicInterfaceInODst(            "split if dynamic interface in ODst" ) );
-	add( new dynamicInterfaceInTSrc(            "split if dynamic interface in TSrc" ) );
-        add( new convertInterfaceIdToStr(    "prepare interface assignments"         ) );
-        add( new PrintRule(                         "generate iptables code"             ) );
-        add( new simplePrintProgress(                                                    ) );
+        add( new AssignInterface("assign rules to interfaces") );
+        add( new dynamicInterfaceInODst("split if dynamic interface in ODst") );
+	add( new dynamicInterfaceInTSrc("set target if dynamic interface in TSrc" ) );
+        add( new convertInterfaceIdToStr("prepare interface assignments") );
+
+        if (fwopt->getBool("use_iptables_restore"))
+        {
+            if (have_dynamic_interfaces)
+                printRule=new PrintRuleIptRstEcho("generate code for iptables-restore using echo");
+            else
+                printRule=new PrintRuleIptRst("generate code for iptables-restore");
+        } else
+            printRule=new PrintRule("generate iptables shell script");
+
+        add( printRule );
+
+        add( new simplePrintProgress() );
 
         runRuleProcessors();
 
@@ -2011,17 +2102,36 @@ void NATCompiler_ipt::compile()
 
 void NATCompiler_ipt::epilog()
 {
-    try {
-
-	output << "#\n#\n";
-
-        output << endl;
-        output << endl;
-
-	cout << _(" NAT rules compiled successfully \n") << flush;
-
-    } catch (FWException ex) {
-	error(ex.toString());
-	exit(1);
+    if (fwopt->getBool("use_iptables_restore"))
+    {
+        output << "#" << endl;
     }
+    cout << _(" NAT rules compiled successfully \n") << flush;
 }
+
+string NATCompiler_ipt::flushAndSetDefaultPolicy()
+{
+    string res="";
+
+/* printRule may be null if there are no NAT rules and we never ran compile() */
+    if(printRule!=NULL)
+    {
+        res += printRule->_flushAndSetDefaultPolicy();
+//    res += printRule->_printOptionalGlobalRules();
+    }
+
+    return res;
+}
+
+string NATCompiler_ipt::commit()
+{
+    string res="";
+
+    if(printRule!=NULL)
+    {
+        res += printRule->_commit();
+    }
+    return res;
+}
+
+
