@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_linux24.cpp,v 1.20 2004/09/28 05:39:07 vkurland Exp $
+  $Id: OSConfigurator_linux24.cpp,v 1.27 2005/05/04 16:39:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -59,10 +59,22 @@ string OSConfigurator_linux24::myPlatformName() { return "Linux24"; }
 
 OSConfigurator_linux24::OSConfigurator_linux24(FWObjectDatabase *_db,
                                                const string &fwname) : 
-    OSConfigurator(_db,fwname) , os_data()
+    OSConfigurator(_db,fwname) , os_data(fw->getStr("host_OS"))
 {
 }
 
+string OSConfigurator_linux24::getInterfaceVarName(FWObject *iface)
+{
+    ostringstream  ostr;
+    string iname=iface->getName();
+    string::size_type p1;
+    while ( (p1=iname.find("."))!=string::npos)
+        iname=iname.replace(p1,1,"_");
+    ostr << "i_" << iname;
+    return ostr.str();
+}
+
+
 void OSConfigurator_linux24::processFirewallOptions() 
 {
     FWOptions* options=fw->getOptionsObject();
@@ -348,7 +360,11 @@ void  OSConfigurator_linux24::configureInterfaces()
 /* if interface name ends with '*', this is a wildcard interface. Do
  * not get its address at this time. */
             if (iface->getName().find("*")==string::npos)
-                output << "getaddr " << iface->getName() << "  i_" << iface->getName() << endl;
+                output << "getaddr "
+                       << iface->getName()
+                       << "  "
+                       << getInterfaceVarName(iface)
+                       << endl;
         }
     }
     output << endl;
@@ -372,30 +388,6 @@ int  OSConfigurator_linux24::prolog()
 
     output << endl;
 
-    output << "$IPTABLES -P OUTPUT  DROP" << endl;
-    output << "$IPTABLES -P INPUT   DROP" << endl;
-    output << "$IPTABLES -P FORWARD DROP" << endl;
-    output << endl;
-
-/*
- *    flush all the rules which existed before
- */
-
-    output << "\n\
-\n\
-cat /proc/net/ip_tables_names | while read table; do\n\
-  test \"X$table\" = \"Xmangle\" && continue\n\
-  $IPTABLES -t $table -L -n | while read c chain rest; do\n\
-      if test \"X$c\" = \"XChain\" ; then\n\
-        $IPTABLES -t $table -F $chain\n\
-      fi\n\
-  done\n\
-  $IPTABLES -t $table -X\n\
-done\n\
-\n";
-
-    output << endl;
-
     configureInterfaces();
 
     return 0;
@@ -405,7 +397,7 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
 {
     FWOptions* options=fw->getOptionsObject();
     
-    string s, path_lsmod, path_modprobe, path_iptables, path_ip, path_logger;
+    string s, path_lsmod, path_modprobe, path_iptables, path_iptables_restore, path_ip, path_logger;
 
     s=options->getStr("linux24_path_lsmod");
     if (!s.empty()) path_lsmod=s;
@@ -419,6 +411,10 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
     if (!s.empty()) path_iptables=s;
     else            path_iptables=os_data.getPathForTool(os,OSData::IPTABLES);
 
+    s=options->getStr("linux24_path_iptables_restore");
+    if (!s.empty()) path_iptables_restore=s;
+    else            path_iptables_restore=os_data.getPathForTool(os,OSData::IPTABLES_RESTORE);
+
     s=options->getStr("linux24_path_ip");
     if (!s.empty()) path_ip=s;
     else            path_ip=os_data.getPathForTool(os,OSData::IP);
@@ -452,7 +448,10 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
     output << "    type=$4"                                << endl;
     output << "    IFS=$OIFS"                              << endl;
     output << ""                                           << endl;
-    output << "    L=`$IP -4 addr ls $dev to $addr | tail -n1`" << endl;
+/*
+ *  see comment about using grep with or without regex below
+ */
+    output << "    L=`$IP -4 addr ls $dev to $addr | grep inet | grep -v :`" << endl;
     output << "    if test -n \"$L\"; then"                << endl;
     output << "      OIFS=$IFS"                            << endl;
     output << "      IFS=\" /\""                           << endl;
@@ -462,21 +461,38 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
     output << "    fi"                                     << endl;
     output << "  fi"                                       << endl;
     output << "  if test -z \"$aadd\"; then"               << endl;
-    output << "    if test \"$type\" = \"POINTOPOINT\"; then"                                        << endl;
+    output << "    if test \"$type\" = \"POINTOPOINT\"; then"<< endl;
     output << "      $IP -4 addr add $addr dev $dev scope global label $dev:FWB${va_num}"            << endl;
-    output << "      va_num=`expr $va_num + 1`"            << endl;
-    output << "    fi"                                     << endl;
-    output << "    if test \"$type\" = \"BROADCAST\"; then"<< endl;
+    output << "      va_num=`expr $va_num + 1`"             << endl;
+    output << "    fi"                                      << endl;
+    output << "    if test \"$type\" = \"BROADCAST\"; then" << endl;
     output << "      $IP -4 addr add $addr/$nm dev $dev brd + scope global label $dev:FWB${va_num}"  << endl;
-    output << "      va_num=`expr $va_num + 1`"            << endl;
-    output << "    fi"                                     << endl;
-    output << "  fi"                                       << endl;
-    output << "}"                                          << endl;
-
+    output << "      va_num=`expr $va_num + 1`"             << endl;
+    output << "    fi"                                      << endl;
+    output << "  fi"                                        << endl;
+    output << "}"                                           << endl;
+    output << endl;
+    output << "getInterfaceVarName() {"                     << endl;
+    output << "  echo $1 | sed 's/\\./_/'"                  << endl;
+    output << "}"                                           << endl;
+    output << endl;
     output << "getaddr() {"                                 << endl;
     output << "  dev=$1"                                    << endl;
     output << "  name=$2"                                   << endl;
-    output << "  L=`$IP -4 addr show dev $dev | tail -n1`"   << endl;
+/*
+ * originally this command looked like this:
+ * $IP -4 addr ls $dev to $addr | grep inet | grep -E \"$dev$\"`
+ *
+ *  i.e. it looked for a line that ends with "$dev":
+ *  inet 10.3.14.40/24 brd 10.3.14.255 scope global eth0  
+ *    as opposed to
+ *  inet 192.168.1.100/24 brd 192.168.1.255 scope global eth0:1
+ *
+ *  It turns out, some busybox-based systems have grep compiled w/o
+ *  support for regular expressions. Using "grep -v :" seems to be an
+ *  easy way to filter out secondary addresses without using regex
+ */
+    output << "  L=`$IP -4 addr show dev $dev | grep inet | grep -v :`"   << endl;
     output << "  test -z \"$L\" && { "                      << endl;
     output << "    eval \"$name=''\""                       << endl;
     output << "    return"                                  << endl;
@@ -494,7 +510,7 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
 
     output << "getinterfaces() {"                           << endl;
     output << "  NAME=$1"                                   << endl;
-    output << "  $IP link show | grep -E \"$NAME[^ ]*: \"| while read L; do" << endl;
+    output << "  $IP link show | grep \": $NAME\" | while read L; do" << endl;
     output << "    OIFS=$IFS"                               << endl;
     output << "    IFS=\" :\""                              << endl;
     output << "    set $L"                                  << endl;
@@ -510,6 +526,7 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
     output << "LSMOD=\""   +path_lsmod+"\"\n";
     output << "MODPROBE=\""+path_modprobe+"\"\n";
     output << "IPTABLES=\""+path_iptables+"\"\n";
+    output << "IPTABLES_RESTORE=\""+path_iptables_restore+"\"\n";
     output << "IP=\""      +path_ip+"\"\n";
     output << "LOGGER=\""  +path_logger+"\"\n";
     output << endl;
@@ -533,6 +550,7 @@ void  OSConfigurator_linux24::printPathForAllTools(const string &os)
     output << "check lsmod    $LSMOD"    << endl;
     output << "check modprobe $MODPROBE" << endl;
     output << "check iptables $IPTABLES" << endl;
+    output << "check iptables-restore $IPTABLES_RESTORE" << endl;
     output << "check ip       $IP"       << endl;
 #endif
 
@@ -543,9 +561,10 @@ void OSConfigurator_linux24::generateCodeForProtocolHandlers(bool have_nat)
 {
     FWOptions* options=fw->getOptionsObject();
 
+    bool nomod=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceBool("/FWBuilderResources/Target/options/suppress_modules");
+
 /* there is no need to load modules on linksys */
-    if (options->getBool("load_modules") &&
-        fw->getStr("host_OS")!="linksys")
+    if (options->getBool("load_modules") && !nomod)
     {
 	output << endl;
 	output << "MODULE_DIR=\"/lib/modules/`uname -r`/kernel/net/ipv4/netfilter/\" " << endl;
