@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRule.cpp,v 1.10 2005/04/14 06:30:37 vkurland Exp $
+  $Id: PolicyCompiler_PrintRule.cpp,v 1.11 2005/05/28 21:16:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -829,14 +829,26 @@ string PolicyCompiler_ipt::PrintRule::_printTimeInterval(PolicyRule *r)
          << setw(2) << setfill('0') << emin  << " ";
 
     if (sdayofweek<0) sdayofweek=0;
-    if (edayofweek<0) edayofweek=6;
+    if (sdayofweek>6) sdayofweek=6;
+    if (edayofweek<0) edayofweek=0;
+    if (edayofweek>6) edayofweek=6;
 
     ostr << " --days ";
     first=true;
-    for (int i=sdayofweek; i<=edayofweek; ++i) {
-        if (!first) ostr << ",";
-        first=false;
-        ostr << daysofweek[i];
+
+    bool inside_interval = false;
+    int  day=0;
+    while (1)
+    {
+        if (!inside_interval && day==sdayofweek) inside_interval=true;
+        if (inside_interval)
+        {
+            if (!first) ostr << ",";
+            first=false;
+            ostr << daysofweek[day];
+            if (day==edayofweek)  break;  // if sdayofweek==edayofweek print one day
+        }
+        if (++day>6) day=0;
     }
 
     return ostr.str();
