@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.cpp,v 1.24 2005/04/14 06:30:37 vkurland Exp $
+  $Id: PolicyCompiler_ipt.cpp,v 1.25 2005/06/11 20:49:46 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -479,7 +479,7 @@ bool PolicyCompiler_ipt::Logging2::processNext()
 	r->setStr("ipt_target",new_chain);
 	r->setLogging(false);
         r->setAction(PolicyRule::Continue);    // ###
-        ruleopt->setInt("limit_value",-1);
+//        ruleopt->setInt("limit_value",-1);
 	tmp_queue.push_back(r);
 
 /*
@@ -509,6 +509,7 @@ bool PolicyCompiler_ipt::Logging2::processNext()
 	r->setLogging(false);
         ruleopt->setBool("stateless",true);
         r->setBool("force_state_check",false);
+        ruleopt->setInt("limit_value",-1);
  	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
@@ -542,6 +543,7 @@ bool PolicyCompiler_ipt::Logging2::processNext()
         ruleopt->setBool("stateless",true);
         r->setBool("force_state_check",false);
         r->setBool("final",true);
+        ruleopt->setInt("limit_value",-1);
 
 	tmp_queue.push_back(r);
     } else
@@ -2971,7 +2973,8 @@ string PolicyCompiler_ipt::debugPrintRule(Rule *r)
         str <<  setw(12) << setfill(' ') << srv.c_str();
         str <<  setw(18) << setfill(' ') << time.c_str();
 
-        if (no==0) {
+        if (no==0)
+        {
             str <<  setw(9)  << setfill(' ') << rule->getActionAsString().c_str();
             str <<  setw(9)  << setfill(' ') << rule->getDirectionAsString().c_str();
             if (rule->getLogging()) str << " LOG";
@@ -2992,6 +2995,10 @@ string PolicyCompiler_ipt::debugPrintRule(Rule *r)
 
     if (rule->getAction()==PolicyRule::Reject)
         s=s+" "+ruleopt->getStr("action_on_reject");
+
+    if (ruleopt!=NULL && ruleopt->getInt("limit_value")>0)
+        s=s+" limit";
+
     return s;
 }
 
