@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipt.cpp,v 1.40 2005/04/10 05:55:20 vkurland Exp $
+  $Id: ipt.cpp,v 1.41 2005/05/29 03:13:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -498,7 +498,14 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         script << "log '";
         script << _("Activating firewall script generated ")
-               << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
+               << timestr << " " << _(" by ")
+/* timezone removed because of bug #1205665 - sometimes timezone name
+ * has "'" in it which confuses shell and causes an error (for
+ * instance French daylight savings time is "Paris, Madrid (heure
+ * d'ete)" where 'e' are actually accented 'e')
+ * 
+ *               << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
+ */
                << user_name;
         script << "'" << endl;
 
