@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: pf.cpp,v 1.23 2005/04/14 06:30:37 vkurland Exp $
+  $Id: pf.cpp,v 1.24 2005/05/29 03:13:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -603,7 +603,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
         fw_file << "log '";
         fw_file << _("Activating firewall script generated ")
-               << timestr << " " << tzname[stm->tm_isdst] << _(" by ")
+               << timestr << " " << _(" by ")
                << user_name;
         fw_file << "'" << endl;
 
