@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipf.cpp,v 1.1 2004/05/11 06:06:29 vkurland Exp $
+  $Id: NATCompiler_ipf.cpp,v 1.2 2005/03/29 08:38:33 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -130,6 +130,62 @@ bool NATCompiler_ipf::VerifyRules::processNext()
     return true;
 }
 
+bool NATCompiler_ipf::ExpandPortRange::processNext()
+{
+    NATRule *rule=getNext(); if (rule==NULL) return false;
+
+    Service          *osrv=compiler->getFirstOSrv(rule);
+
+    if (UDPService::isA(osrv) || TCPService::isA(osrv))
+    {
+      int rs = osrv->getInt("dst_range_start");
+      int re = osrv->getInt("dst_range_end");
+
+      int numPorts = re-rs+1;
+      if (numPorts==1)
+      {
+          tmp_queue.push_back(rule);
+	  return true;
+      }
+
+      if (numPorts > 20)
+      {
+	ostringstream ostr;
+	ostr << string("Expanding port range ") << osrv->getName()
+	     << " yields " << numPorts << " rules";
+	compiler->warning(ostr.str());
+      }
+
+      string newSrvType = TCPService::TYPENAME;
+      if (UDPService::isA(osrv)) newSrvType = UDPService::TYPENAME;
+
+      for (int p=rs; p<=re; ++p)
+      {
+	  NATRule *r = NATRule::cast(
+                compiler->dbcopy->create(NATRule::TYPENAME,false) );
+	  r->duplicate(rule);
+
+	  FWObject *newSrv = compiler->dbcopy->create(newSrvType,false);
+	  newSrv->duplicate(osrv,true);
+	  newSrv->setInt("dst_range_start",p);
+	  newSrv->setInt("dst_range_end",p);
+	  compiler->dbcopy->add(newSrv,false);
+	  compiler->cacheObj(newSrv);
+
+	  RuleElementOSrv *nosrv = r->getOSrv();
+	  nosrv->clearChildren();
+	  nosrv->addRef(newSrv);
+
+	  compiler->temp_ruleset->add(r);
+	  tmp_queue.push_back(r);
+      }
+    } else
+    {
+      tmp_queue.push_back(rule);
+    }
+    return true;
+}
+
 bool NATCompiler_ipf::AssignInterface::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
@@ -241,6 +297,7 @@ bool NATCompiler_ipf::AssignInterface::processNext()
 
         return true;
     }
+    default: ;
     }
 
     throw FWException(_("Could not assign NAT rule to the interface. Perhaps one of the \n\
@@ -444,6 +501,7 @@ void NATCompiler_ipf::compile()
         add( new eliminateDuplicatesInOSRV(  "eliminate duplicates in OSRV"              ) );
 
         add( new splitOnOSrv( "split rule on original service" ) );
+	add( new ExpandPortRange("expand port ranges") );
         add( new fillTranslatedSrv( "fill translated service" ) );
         add( new NATRuleType( "determine NAT rule types" ) );
         add( new VerifyRules( "verify NAT rules" ) );
