@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf.cpp,v 1.3 2004/09/19 06:20:12 vkurland Exp $
+  $Id: NATCompiler_pf.cpp,v 1.3.4.1 2005/09/05 05:31:23 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -254,8 +254,15 @@ bool NATCompiler_pf::VerifyRules::processNext()
     if (osrv->getNeg())
         throw FWException(_("Negation in original service is not supported. Rule ")+rule->getLabel());
 
+    /* bug #1276083: "Destination NAT rules". this restriction is not
+     * true at least as of OpenBSD 3.5
+     *
     if (rule->getRuleType()==NATRule::DNAT && osrv->isAny()) 
 	throw FWException(_("Service must be specified for destination translation rule. Rule ")+rule->getLabel());
+    */
+
+    if (rule->getRuleType()==NATRule::DNAT && osrv->isAny() && !tsrv->isAny())
+	throw FWException(_("Can not translate 'any' into a specific service. Rule ")+rule->getLabel());
 
     if (tsrc->getNeg())
         throw FWException(_("Can not use negation in translated source. Rule ")+rule->getLabel());
