@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf_writers.cpp,v 1.3 2004/07/05 20:40:26 vkurland Exp $
+  $Id: NATCompiler_pf_writers.cpp,v 1.4 2005/01/28 07:54:58 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,6 +32,7 @@
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
+#include "fwbuilder/CustomService.h"
 #include "fwbuilder/Host.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Interface.h"
@@ -179,9 +180,12 @@ bool NATCompiler_pf::PrintRule::processNext()
 
 void NATCompiler_pf::PrintRule::_printProtocol(Service *srv)
 {
-    string s=srv->getProtocolName();
-    if (s=="ip") s="{tcp udp icmp}";
-    compiler->output << "proto " <<  s << " ";
+    if ( ! CustomService::isA(srv))
+    {
+        string s=srv->getProtocolName();
+        if (s=="ip") s="{tcp udp icmp}";
+        compiler->output << "proto " <<  s << " ";
+    }
 }
 
 void NATCompiler_pf::PrintRule::_printPort(Service *srv)
