@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipf_writers.cpp,v 1.3 2004/05/15 06:51:07 vkurland Exp $
+  $Id: PolicyCompiler_ipf_writers.cpp,v 1.4 2005/03/30 07:33:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -35,6 +35,7 @@
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/FWOptions.h"
+#include "fwbuilder/Firewall.h"
 
 #include <iostream>
 
@@ -220,9 +221,19 @@ void PolicyCompiler_ipf::PrintRule::_printWith(libfwbuilder::Service *srv)
  */
 void PolicyCompiler_ipf::PrintRule::_printAddr(Address  *o,bool neg)
 {
+    FWOptions* options=compiler->fw->getOptionsObject();
+
     IPAddress addr=o->getAddress();
     Netmask   mask=o->getNetmask();
 
+    if (options->getBool("dynAddr") &&
+        Interface::cast(o)!=NULL && Interface::cast(o)->isDyn()) 
+    {
+        if (neg) compiler->output << "! ";
+        compiler->output << "(" << o->getName() << ") ";
+        return;
+    }
+
     if (Interface::cast(o)!=NULL) {
 	mask=Netmask("255.255.255.255");
     }
