@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.cpp,v 1.1 2004/05/11 06:06:29 vkurland Exp $
+  $Id: PolicyCompiler_ipfw.cpp,v 1.4 2005/04/04 07:00:54 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -51,7 +51,6 @@ int PolicyCompiler_ipfw::prolog()
 {
     int n= PolicyCompiler_pf::prolog();
 
-    FWObject    *grp;
     anytcp=TCPService::cast(dbcopy->create(TCPService::TYPENAME,false) );
     anytcp->setId(ANY_TCP_OBJ_ID);
     dbcopy->add(anytcp,false);
@@ -230,6 +229,133 @@ bool PolicyCompiler_ipfw::doSrvNegation::processNext()
     return true;
 }
 
+bool PolicyCompiler_ipfw::separatePortRanges::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElementSrv *rel= rule->getSrv();
+
+    if (rel->size()==1) 
+    {
+	tmp_queue.push_back(rule);
+	return true;
+    }
+
+    list<Service*> services;   
+    bool sawServiceWithPortRange=false;
+    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
+    {	    
+	FWObject *o= *i;
+	if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	Service *s=Service::cast(o);
+	assert(s!=NULL);
+
+	if ( TCPService::isA(s) || UDPService::isA(s) ) 
+        {
+            unsigned srs=s->getInt("src_range_start");
+            unsigned sre=s->getInt("src_range_end");
+            unsigned drs=s->getInt("dst_range_start");
+            unsigned dre=s->getInt("dst_range_end");
+
+            if (srs!=0 && sre==0) sre=srs;
+            if (drs!=0 && dre==0) dre=drs;
+
+            if (srs!=sre || drs!=dre)
+            {
+                /* leave the very first service with port range in this rule,
+                 * split others into separate rules
+                 */
+                if (sawServiceWithPortRange)
+                {
+                    PolicyRule *r= PolicyRule::cast(
+                        compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                    compiler->temp_ruleset->add(r);
+                    r->duplicate(rule);
+                    RuleElementSrv *nsrv=r->getSrv();
+                    nsrv->clearChildren();
+                    nsrv->addRef( s );
+                    tmp_queue.push_back(r);
+                    services.push_back(s);
+                }
+                sawServiceWithPortRange=true;
+            } 
+        }
+    }
+    for (list<Service*>::iterator i=services.begin(); i!=services.end(); i++) 
+	rel->removeRef( (*i) );
+
+    if (!rel->isAny())
+	tmp_queue.push_back(rule);
+
+    return true;
+}
+
+bool PolicyCompiler_ipfw::sortTCPUDPServices::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElementSrv *rel= rule->getSrv();
+
+    if (rel->size()==1) 
+    {
+	tmp_queue.push_back(rule);
+	return true;
+    }
+
+    FWObject *o=rel->front();
+    if (o && FWReference::cast(o)!=NULL)
+        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+
+    Service *s1= Service::cast(o);
+    if ( !UDPService::isA(s1) && !TCPService::isA(s1))
+    {
+	tmp_queue.push_back(rule);
+	return true;
+    }
+
+/*
+ * we know that at this point if there the original rule had service
+ * objects with port ranges, there is only one left. We just need to
+ * move it to the front of the list.
+ */
+    Service *portRangeSvc=NULL;
+    for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) 
+    {	    
+	FWObject *o= *i;
+	if (FWReference::cast(o)!=NULL)
+            o=compiler->getCachedObj(o->getStr("ref"));
+	Service *s=Service::cast(o);
+	assert(s!=NULL);
+
+        unsigned srs=s->getInt("src_range_start");
+        unsigned sre=s->getInt("src_range_end");
+        unsigned drs=s->getInt("dst_range_start");
+        unsigned dre=s->getInt("dst_range_end");
+
+        if (srs!=0 && sre==0) sre=srs;
+        if (drs!=0 && dre==0) dre=drs;
+
+        if (srs!=sre || drs!=dre)
+        {
+            portRangeSvc=s;
+            break;
+        } 
+    }
+
+    if (portRangeSvc)
+    {
+        rel->removeRef(portRangeSvc);
+
+/* It certainly would have been better if we had FWObject::insertRef() */
+        FWReference *oref = portRangeSvc->createRef();
+        portRangeSvc->ref();
+
+        rel->push_front(oref);
+        oref->setParent(rel);
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 void PolicyCompiler_ipfw::specialCaseWithDynInterface::dropDynamicInterface(RuleElement *re)
 {
     list<FWObject*> cl;
@@ -269,8 +395,6 @@ PolicyCompiler_ipfw::calculateNum::calculateNum(const std::string &n) : PolicyRu
 
 bool PolicyCompiler_ipfw::calculateNum::processNext()
 {
-    PolicyRule *rule;
-
     slurp();
     if (tmp_queue.size()==0) return false;
 
@@ -310,7 +434,6 @@ bool PolicyCompiler_ipfw::calculateNum::processNext()
     return true;
 }
 
-
 bool PolicyCompiler_ipfw::checkForKeepState::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
@@ -367,74 +490,84 @@ void PolicyCompiler_ipfw::compile()
 	addDefaultPolicyRule();
         bool check_for_recursive_groups=true;
 
-        if ( fw->getOptionsObject()->getBool ("check_shading") ) 
+        if ( fw->getOptionsObject()->getBool("check_shading")) 
         {
-            add( new Begin                       (" Detecting rule shadowing"                ) );
-            add( new printTotalNumberOfRules     (                                         ) );
+            add( new Begin(" Detecting rule shadowing"));
+            add( new printTotalNumberOfRules(                                         ));
 
-            add( new recursiveGroupsInSrc(       "check for recursive groups in SRC"     ) );
-            add( new recursiveGroupsInDst(       "check for recursive groups in DST"     ) );
-            add( new recursiveGroupsInSrv(       "check for recursive groups in SRV"     ) );
+            add( new recursiveGroupsInSrc("check for recursive grps in SRC"));
+            add( new recursiveGroupsInDst("check for recursive grps in DST"));
+            add( new recursiveGroupsInSrv("check for recursive grps in SRV"));
             check_for_recursive_groups=false;
 
-            add( new ExpandGroups                ("expand groups"                          ) );
-            add( new eliminateDuplicatesInSRC    ("eliminate duplicates in SRC"            ) );
-            add( new eliminateDuplicatesInDST    ("eliminate duplicates in DST"            ) );
-            add( new eliminateDuplicatesInSRV    ("eliminate duplicates in SRV"            ) );
-            add( new ExpandMultipleAddressesInSRC("expand objects with multiple addresses in SRC" ) );
-            add( new ExpandMultipleAddressesInDST("expand objects with multiple addresses in DST" ) );
-            add( new ConvertToAtomic             ("convert to atomic rules"                ) );
-            add( new DetectShadowing             ("Detect shadowing"                         ) );
-            add( new simplePrintProgress         (                                         ) );
+            add( new ExpandGroups("expand groups"));
+            add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
+            add( new eliminateDuplicatesInDST("eliminate duplicates in DST"));
+            add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
+            add( new ExpandMultipleAddressesInSRC(
+                     "expand objects with multiple addresses in SRC"));
+            add( new ExpandMultipleAddressesInDST(
+                     "expand objects with multiple addresses in DST"));
+            add( new ConvertToAtomic("convert to atomic rules"));
+            add( new DetectShadowing("Detect shadowing"));
+            add( new simplePrintProgress(                                         ));
 
             runRuleProcessors();
             deleteRuleProcessors();
         }
 
 
-        add( new Begin(                       "Begin processing"                                ) );
-        add( new printTotalNumberOfRules() );
+        add( new Begin("Begin processing"));
+        add( new printTotalNumberOfRules());
 
         if (check_for_recursive_groups)
         {
-            add( new recursiveGroupsInSrc(   "check for recursive groups in SRC"     ) );
-            add( new recursiveGroupsInDst(   "check for recursive groups in DST"     ) );
-            add( new recursiveGroupsInSrv(   "check for recursive groups in SRV"     ) );
+            add( new recursiveGroupsInSrc("check for recursive grps in SRC"));
+            add( new recursiveGroupsInDst("check for recursive grps in DST"));
+            add( new recursiveGroupsInSrv("check for recursive grps in SRV"));
         }
 
-        add( new emptyGroupsInSrc(           "check for empty groups in SRC"         ) );
-        add( new emptyGroupsInDst(           "check for empty groups in DST"         ) );
-        add( new emptyGroupsInSrv(           "check for empty groups in SRV"         ) );
+        add( new emptyGroupsInSrc("check for empty grps in SRC"));
+        add( new emptyGroupsInDst("check for empty grps in DST"));
+        add( new emptyGroupsInSrv("check for empty grps in SRV"));
 
-        add( new doSrcNegation(               "process negation in Src"                         ) );
-        add( new doDstNegation(               "process negation in Dst"                         ) );
-        add( new doSrvNegation(               "process negation in Srv"                         ) );
-        add( new ExpandGroups(                "expand groups"                                   ) );
-        add( new eliminateDuplicatesInSRC(    "eliminate duplicates in SRC"                     ) );
-        add( new eliminateDuplicatesInDST(    "eliminate duplicates in DST"                     ) );
-        add( new eliminateDuplicatesInSRV(    "eliminate duplicates in SRV"                     ) );
+        add( new doSrcNegation("process negation in Src"));
+        add( new doDstNegation("process negation in Dst"));
+        add( new doSrvNegation("process negation in Srv"));
+        add( new ExpandGroups("expand groups"));
+        add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
+        add( new eliminateDuplicatesInDST("eliminate duplicates in DST"));
+        add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
 
-        add( new splitIfFirewallInSrc(        "split rule if firewall is in Src"                ) );
-        add( new splitIfFirewallInDst(        "split rule if firewall is in Dst"                ) );
-        add( new fillDirection(               "determine directions"                            ) );
-        add( new ExpandMultipleAddresses(     "expand objects with multiple addresses"          ) );
-        add( new checkForDynamicInterfacesOfOtherObjects( "check for dynamic interfaces of other hosts and firewalls" ) );
-        add( new MACFiltering(                "verify for MAC address filtering"                ) );
-        add( new checkForUnnumbered(          "check for unnumbered interfaces"                 ) );
-        add( new specialCaseWithDynInterface( "check for a special cases with dynamic interface") );
-        add( new addressRanges(               "expand address range objects"                    ) );
-        add( new splitServices(               "split rules with different protocols"            ) );
-        add( new separateTCPWithFlags(        "separate TCP services with flags"                ) );
-        add( new verifyCustomServices(        "verify custom services for this platform"        ) );
-        add( new SpecialServices(             "check for special services"                      ) );
-//        add( new expandAnyService(            "expand ANY service for stateful rules"           ) );
-        add( new ConvertToAtomicForAddresses( "convert to atomic rules in SRC and DST"          ) );
-        add( new checkForZeroAddr(            "check for zero addresses"                        ) );
+        add( new splitIfFirewallInSrc("split rule if firewall is in Src"));
+        add( new splitIfFirewallInDst("split rule if firewall is in Dst"));
+        add( new fillDirection("determine directions"));
+        add( new ExpandMultipleAddresses(
+                 "expand objects with multiple addresses"));
+        add( new checkForDynamicInterfacesOfOtherObjects(
+                 "check for dynamic interfaces of other hosts and firewalls"));
+        add( new MACFiltering("verify for MAC address filtering"));
+        add( new checkForUnnumbered("check for unnumbered interfaces"));
+        add( new specialCaseWithDynInterface(
+                 "check for a special cases with dynamic interface"));
+        add( new addressRanges("expand address range objects"));
+        add( new splitServices("split rules with different protocols"));
+        add( new separateTCPWithFlags("separate TCP services with flags"));
+        add( new separateSrcPort("split on TCP and UDP with source ports"));
+        add( new separatePortRanges("split services with port ranges"));
+        add( new sortTCPUDPServices("move port ranges to the front of ports"));
+        add( new verifyCustomServices(
+                 "verify custom services for this platform"));
+        add( new SpecialServices("check for special services"));
+//        add( new expandAnyService("expand ANY service for stateful rules"));
+        add( new ConvertToAtomicForAddresses(
+                 "convert to atomic rules in SRC and DST"));
+        add( new checkForZeroAddr("check for zero addresses"));
 
-	add( new calculateNum(                "calculate rule numbers "                         ) );
-        add( new convertInterfaceIdToStr(    "prepare interface assignments"         ) );
-        add( new PrintRule(                   "generate ipf code"                               ) );
-        add( new simplePrintProgress() );
+	add( new calculateNum("calculate rule numbers "));
+        add( new convertInterfaceIdToStr("prepare interface assignments"));
+        add( new PrintRule("generate ipf code"));
+        add( new simplePrintProgress());
 
         runRuleProcessors();
 
@@ -448,7 +581,6 @@ void PolicyCompiler_ipfw::compile()
 string PolicyCompiler_ipfw::debugPrintRule(Rule *r)
 {
     PolicyRule *rule=PolicyRule::cast(r);
-    FWOptions  *ruleopt =rule->getOptionsObject();
 
     string  iface = rule->getInterfaceId();
     if (iface!="") {
