@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.5 2004/12/31 07:31:45 vkurland Exp $
+  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.7 2005/05/30 22:09:19 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -404,7 +404,7 @@ PolicyCompiler_ipfw::PrintRule::PrintRule(const std::string &name) : PolicyCompi
 bool PolicyCompiler_ipfw::PrintRule::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    FWOptions*  options = compiler->fw->getOptionsObject();
+//    FWOptions*  options = compiler->fw->getOptionsObject();
     FWOptions  *ruleopt = rule->getOptionsObject();
 
     tmp_queue.push_back(rule);
@@ -417,7 +417,10 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
     if (print_once_on_top) 
     {
         compiler->output << quote << "$IPFW" << quote
-                         << " add 1 check-state ip from any to any"  << endl;
+                         << " set disable 1" << endl;
+
+        compiler->output << quote << "$IPFW" << quote
+                         << " add 1 set 1 check-state ip from any to any"  << endl;
         compiler->output << endl;
 
         print_once_on_top=false;
@@ -452,8 +455,36 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
     RuleElementSrv *srvrel=rule->getSrv();
     Service        *srv   =compiler->getFirstSrv(rule);  assert(srv);
 
+    if (rule->getBool("needs_established"))
+    {
+// ipfw_num is assigned with a step of 10, so it is safe to substract 1 here
+        compiler->output <<  quote << "$IPFW" << quote
+                         << " add " << rule->getInt("ipfw_num")-1 << " set 1 ";
+
+        _printAction(rule);
+
+        if (rule->getLogging())       compiler->output << " log ";
+
+        _printProtocol(srv);
+
+        compiler->output << " from ";
+        _printSrcAddr(srcrel);
+        _printSrcService(srvrel);
+
+        compiler->output << " to ";
+        _printDstAddr(dstrel);
+        _printDstService(srvrel);
+
+        _printDirection(rule);
+        _printInterface(rule);
+
+	compiler->output << "established ";
+
+        compiler->output << endl;
+    }
+
     compiler->output <<  quote << "$IPFW" << quote
-                     << " add " << rule->getInt("ipfw_num") << " ";
+                     << " add " << rule->getInt("ipfw_num") << " set 1 ";
 
     _printAction(rule);
 
@@ -492,6 +523,7 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
 	compiler->output << "keep-state ";
     }
 
+    compiler->output << " || exit 1" << endl;
     compiler->output << endl;
 
     return true;
