@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp,v 1.7 2005/04/04 07:00:54 vkurland Exp $
+  $Id: PolicyCompiler_pf.cpp,v 1.8 2005/05/30 22:09:19 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -37,6 +37,7 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Network.h"
 
 #include <algorithm>
 #include <functional>
@@ -212,8 +213,41 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
         dbcopy->add(ssh,false);
         cacheObj(ssh); // to keep cache consistent
 
-        IPv4 *mgmt_workstation = IPv4::cast(dbcopy->create(IPv4::TYPENAME,false));
-        mgmt_workstation->setAddress( getCachedFwOpt()->getStr("mgmt_addr") );
+        string mgmt_addr = getCachedFwOpt()->getStr("mgmt_addr");
+        IPAddress  addr;
+        Netmask netmask(32);
+        try
+        {
+            addr = IPAddress(mgmt_addr);
+            string::size_type sep = mgmt_addr.find("/");
+            if (sep != string::npos)
+            {
+                addr = IPAddress(mgmt_addr.substr(0,sep));
+                string nm = mgmt_addr.substr(sep+1);
+                int o1,o2,o3,o4;
+                if(sscanf(nm.c_str(), "%3u.%3u.%3u.%3u", &o1, &o2, &o3, &o4)==4)
+                {
+                    netmask = Netmask(nm);
+                } else
+                {
+                    sscanf(nm.c_str(),"%u",&o1);
+                    netmask = Netmask(o1);
+                }
+            }
+        } catch(FWException &ex)
+        {
+            char errstr[256];
+            sprintf(errstr,
+                    _("Invalid address for the backup ssh access: '%s'"),
+                    mgmt_addr.c_str() );
+            abort( errstr );
+        }
+
+        Network *mgmt_workstation = Network::cast(dbcopy->create(Network::TYPENAME,false));
+        mgmt_workstation->setAddress( addr );
+        mgmt_workstation->setNetmask( netmask );
+//        IPv4 *mgmt_workstation = IPv4::cast(dbcopy->create(IPv4::TYPENAME,false));
+//        mgmt_workstation->setAddress( getCachedFwOpt()->getStr("mgmt_addr") );
         dbcopy->add(mgmt_workstation,false);
         cacheObj(mgmt_workstation); // to keep cache consistent
 
@@ -228,6 +262,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
         r->setHidden(true);
         r->setFallback(false);
         r->setLabel("backup ssh access rule");
+        r->setBool("needs_established",true);  // supported in ipfw
 
         RuleElement *src=r->getSrc();
         assert(src!=NULL);
